@@ -1,7 +1,7 @@
 #!/bin/sh
-srcversion='$MirOS: src/bin/mksh/Build.sh,v 1.477 2011/04/09 21:00:58 tg Exp $'
+srcversion='$MirOS: src/bin/mksh/Build.sh,v 1.434 2010/01/01 18:27:42 tg Exp $'
 #-
-# Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009
 #	Thorsten Glaser <tg@mirbsd.org>
 #
 # Provided that these terms and disclaimer and all copyright notices
@@ -19,19 +19,13 @@ srcversion='$MirOS: src/bin/mksh/Build.sh,v 1.477 2011/04/09 21:00:58 tg Exp $'
 # damage or existence of a defect, except proven that it results out
 # of said person's immediate fault when using the work as intended.
 #-
-# People analysing the output must whitelist conftest.c for any kind
-# of compiler warning checks (mirtoconf is by design not quiet).
-#
 # Environment used:	CC CFLAGS CPPFLAGS LDFLAGS LIBS NOWARN NROFF
 #			TARGET_OS TARGET_OSREV
 # Feature selectors:	USE_PRINTF_BUILTIN
 # CPPFLAGS recognised:	MKSH_ASSUME_UTF8 MKSH_BINSHREDUCED MKSH_CLS_STRING
 #			MKSH_CONSERVATIVE_FDS MKSH_MIDNIGHTBSD01ASH_COMPAT
 #			MKSH_NOPWNAM MKSH_NO_LIMITS MKSH_SMALL MKSH_S_NOVI
-#			MKSH_UNEMPLOYED MKSH_DEFAULT_EXECSHELL MKSHRC_PATH
-#			MKSH_DEFAULT_TMPDIR MKSH_CLRTOEOL_STRING MKSH_A4PB
-#			MKSH_NO_DEPRECATED_WARNING MKSH_DONT_EMIT_IDSTRING
-#			MKSH_NOPROSPECTOFWORK
+#			MKSH_UNEMPLOYED
 
 LC_ALL=C
 export LC_ALL
@@ -53,15 +47,6 @@ vq() {
 	eval "$@"
 }
 
-rmf() {
-	for _f in "$@"; do
-		case ${_f} in
-		mksh.1) ;;
-		*) rm -f "${_f}" ;;
-		esac
-	done
-}
-
 if test -d /usr/xpg4/bin/. >/dev/null 2>&1; then
 	# Solaris: some of the tools have weird behaviour, use portable ones
 	PATH=/usr/xpg4/bin:$PATH
@@ -164,11 +149,12 @@ ac_testn() {
 		fr=0
 	fi
 	ac_testinit "$@" || return
-	cat >conftest.c
-	vv ']' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN conftest.c $LIBS $ccpr"
+	cat >scn.c
+	vv ']' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN scn.c $LIBS $ccpr" | \
+	    sed 's^\] scn.c:\([0-9]*\):\] mirtoconf(\1):'
 	test $tcfn = no && test -f a.out && tcfn=a.out
 	test $tcfn = no && test -f a.exe && tcfn=a.exe
-	test $tcfn = no && test -f conftest && tcfn=conftest
+	test $tcfn = no && test -f scn && tcfn=scn
 	if test -f $tcfn; then
 		test 1 = $fr || fv=1
 	else
@@ -179,10 +165,9 @@ ac_testn() {
 		test $ct = gcc && vscan='unrecogni[sz]ed'
 		test $ct = hpcc && vscan='unsupported'
 		test $ct = pcc && vscan='unsupported'
-		test $ct = sunpro && vscan='-e ignored -e turned.off'
 	fi
-	test -n "$vscan" && grep $vscan vv.out >/dev/null 2>&1 && fv=$fr
-	rmf conftest.c conftest.o ${tcfn}* vv.out
+	test -n "$vscan" && grep "$vscan" vv.out >/dev/null 2>&1 && fv=$fr
+	rm -f scn.c scn.o ${tcfn}* vv.out
 	ac_testdone
 }
 
@@ -262,7 +247,7 @@ ac_header() {
 	echo "#include <$hf>" >>x
 	echo 'int main(void) { return (0); }' >>x
 	ac_testn "$hv" "" "<$hf>" <x
-	rmf x
+	rm -f x
 	test 1 = $na || ac_cppflags
 }
 
@@ -285,95 +270,50 @@ if test -d mksh || test -d mksh.exe; then
 	echo "$me: Error: ./mksh is a directory!" >&2
 	exit 1
 fi
-rmf a.exe* a.out* conftest.c *core lft mksh* no *.bc *.ll *.o \
-    Rebuild.sh signames.inc test.sh x vv.out
+rm -f a.exe* a.out* *core crypt.exp lft mksh mksh.cat1 mksh.exe mksh.s \
+    no *.o scn.c signames.inc stdint.h test.sh x vv.out
 
 curdir=`pwd` srcdir=`dirname "$0"` check_categories=
-test -n "$dirname" || dirname=.
-dstversion=`sed -n '/define MKSH_VERSION/s/^.*"\(.*\)".*$/\1/p' $srcdir/sh.h`
 
 e=echo
 r=0
 eq=0
 pm=0
 cm=normal
-optflags=-std-compile-opts
-last=
+llvm=
 
 for i
 do
-	case $last:$i in
-	c:combine|c:dragonegg|c:llvm)
-		cm=$i
-		last=
-		;;
-	c:*)
-		echo "$me: Unknown option -c '$i'!" >&2
-		exit 1
-		;;
-	o:*)
-		optflags=$i
-		last=
-		;;
-	:-c)
-		last=c
-		;;
-	:-combine)
-		cm=combine
-		echo "$me: Warning: '$i' is deprecated, use '-c combine' instead!" >&2
-		;;
-	:-g)
-		# checker, debug, valgrind build
-		CPPFLAGS="$CPPFLAGS -DDEBUG"
-		CFLAGS="$CFLAGS -g3 -fno-builtin"
-		;;
-	:-j)
+	case $i in
+	-j)
 		pm=1
 		;;
-	:-llvm)
-		cm=llvm
-		optflags=-std-compile-opts
-		echo "$me: Warning: '$i' is deprecated, use '-c llvm -O' instead!" >&2
+	-combine)
+		cm=combine
 		;;
-	:-llvm=*)
+	-llvm)
 		cm=llvm
-		optflags=`echo "x$i" | sed 's/^x-llvm=//'`
-		echo "$me: Warning: '$i' is deprecated, use '-c llvm -o $llvm' instead!" >&2
+		llvm=-std-compile-opts
 		;;
-	:-M)
+	-llvm=*)
+		cm=llvm
+		llvm=`echo "x$i" | sed 's/^x-llvm=//'`
+		;;
+	-M)
 		cm=makefile
 		;;
-	:-O)
-		optflags=-std-compile-opts
-		;;
-	:-o)
-		last=o
-		;;
-	:-Q)
+	-Q)
 		eq=1
 		;;
-	:-r)
+	-r)
 		r=1
 		;;
-	:-v)
-		echo "Build.sh $srcversion"
-		echo "for mksh $dstversion"
-		exit 0
-		;;
-	:*)
-		echo "$me: Unknown option '$i'!" >&2
-		exit 1
-		;;
 	*)
-		echo "$me: Unknown option -'$last' '$i'!" >&2
+		echo "$me: Unknown option '$i'!" >&2
 		exit 1
 		;;
 	esac
 done
-if test -n "$last"; then
-	echo "$me: Option -'$last' not followed by argument!" >&2
-	exit 1
-fi
 
 SRCS="lalloc.c edit.c eval.c exec.c expr.c funcs.c histrap.c"
 SRCS="$SRCS jobs.c lex.c main.c misc.c shf.c syn.c tree.c var.c"
@@ -389,13 +329,7 @@ oswarn=
 ccpc=-Wc,
 ccpl=-Wl,
 tsts=
-ccpr='|| for _f in ${tcfn}*; do test x"${_f}" = x"mksh.1" || rm -f "${_f}"; done'
-
-# Evil hack
-if test x"$TARGET_OS" = x"Android"; then
-	check_categories="$check_categories android"
-	TARGET_OS=Linux
-fi
+ccpr='|| rm -f ${tcfn}*'
 
 # Configuration depending on OS revision, on OSes that need them
 case $TARGET_OS in
@@ -408,9 +342,17 @@ esac
 case $TARGET_OS in
 AIX)
 	CPPFLAGS="$CPPFLAGS -D_ALL_SOURCE"
+	if test x"$LDFLAGS" = x""; then
+		LDFLAGS="${ccpl}-bI:crypt.exp"
+		echo '#!
+__crypt_r
+__encrypt_r
+__setkey_r' >crypt.exp
+	fi
+	: ${LIBS='-lcrypt'}
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
-BeOS)
+BeOS|Haiku)
 	oswarn=' and will currently not work'
 	;;
 BSD/OS)
@@ -425,11 +367,6 @@ DragonFly)
 	;;
 FreeBSD)
 	;;
-FreeMiNT)
-	oswarn="; it has minor issues"
-	CPPFLAGS="$CPPFLAGS -D_GNU_SOURCE"
-	: ${HAVE_SETLOCALE_CTYPE=0}
-	;;
 GNU)
 	# define NO_PATH_MAX to use Hurd-only functions
 	CPPFLAGS="$CPPFLAGS -D_GNU_SOURCE -DNO_PATH_MAX"
@@ -437,9 +374,6 @@ GNU)
 GNU/kFreeBSD)
 	CPPFLAGS="$CPPFLAGS -D_GNU_SOURCE"
 	;;
-Haiku)
-	CPPFLAGS="$CPPFLAGS -DMKSH_ASSUME_UTF8"
-	;;
 HP-UX)
 	;;
 Interix)
@@ -453,7 +387,7 @@ IRIX*)
 	: ${HAVE_SETLOCALE_CTYPE=0}
 	;;
 Linux)
-	CPPFLAGS="$CPPFLAGS -D_GNU_SOURCE -DSETUID_CAN_FAIL_WITH_EAGAIN"
+	CPPFLAGS="$CPPFLAGS -D_GNU_SOURCE"
 	: ${HAVE_REVOKE=0}
 	;;
 MidnightBSD)
@@ -466,10 +400,6 @@ Minix)
 	;;
 MirBSD)
 	;;
-MSYS_*)
-	# probably same as CYGWIN* – need to test; from RT|Chatzilla
-	oswarn='but will probably work'
-	;;
 NetBSD)
 	;;
 OpenBSD)
@@ -526,13 +456,12 @@ UWIN*)
 	;;
 esac
 
-: ${HAVE_MKNOD=0}
-
 : ${CC=cc} ${NROFF=nroff}
 test 0 = $r && echo | $NROFF -v 2>&1 | grep GNU >/dev/null 2>&1 && \
     NROFF="$NROFF -c"
 
 # this aids me in tracing FTBFSen without access to the buildd
+dstversion=`sed -n '/define MKSH_VERSION/s/^.*"\(.*\)".*$/\1/p' $srcdir/sh.h`
 $e "Hi from$ao $bi$srcversion$ao on:"
 case $TARGET_OS in
 Darwin)
@@ -574,7 +503,7 @@ $e $bi$me: Scanning for functions... please ignore any errors.$ao
 # - nwcc defines __GNUC__ too
 CPP="$CC -E"
 $e ... which compiler seems to be used
-cat >conftest.c <<'EOF'
+cat >scn.c <<'EOF'
 #if defined(__ICC) || defined(__INTEL_COMPILER)
 ct=icc
 #elif defined(__xlC__) || defined(__IBMC__)
@@ -630,11 +559,12 @@ ct=unknown
 #endif
 EOF
 ct=unknown
-vv ']' "$CPP $CFLAGS $CPPFLAGS $NOWARN conftest.c | grep ct= | tr -d \\\\015 >x"
+vv ']' "$CPP $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN scn.c $LIBS | \
+    grep ct= | tr -d \\\\015 >x"
 sed 's/^/[ /' x
 eval `cat x`
-rmf x vv.out
-echo 'int main(void) { return (0); }' >conftest.c
+rm -f x vv.out
+echo 'int main(void) { return (0); }' >scn.c
 case $ct in
 ack)
 	# work around "the famous ACK const bug"
@@ -664,7 +594,7 @@ clang)
 	;;
 dec)
 	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN $LIBS -V"
-	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -Wl,-V conftest.c $LIBS"
+	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -Wl,-V scn.c $LIBS"
 	;;
 dmc)
 	echo >&2 "Warning: Digital Mars Compiler detected. When running under"
@@ -673,13 +603,13 @@ dmc)
 	echo >&2 "    please report success/failure to the developers."
 	;;
 gcc)
-	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -v conftest.c $LIBS"
+	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -v scn.c $LIBS"
 	vv '|' 'echo `$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN $LIBS \
 	    -dumpmachine` gcc`$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN \
 	    $LIBS -dumpversion`'
 	;;
 hpcc)
-	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -V conftest.c $LIBS"
+	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -V scn.c $LIBS"
 	;;
 iar)
 	echo >&2 'Warning: IAR Systems (http://www.iar.com) compiler for embedded
@@ -735,7 +665,7 @@ sdcc)
     own risk, please report success/failure to the developers.'
 	;;
 sunpro)
-	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -V conftest.c $LIBS"
+	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -V scn.c $LIBS"
 	;;
 tcc)
 	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN $LIBS -v"
@@ -746,7 +676,7 @@ tendra)
 	;;
 ucode)
 	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN $LIBS -V"
-	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -Wl,-V conftest.c $LIBS"
+	vv '|' "$CC $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -Wl,-V scn.c $LIBS"
 	;;
 watcom)
 	echo >&2 'Warning: Watcom C Compiler detected. This compiler has not yet
@@ -761,13 +691,9 @@ xlc)
 	ct=unknown
 	;;
 esac
-case $cm in
-dragonegg|llvm)
-	vv '|' "llc -version"
-	;;
-esac
+test $cm = llvm && vv '|' "llc -version"
 $e "$bi==> which compiler seems to be used...$ao $ui$ct$ao"
-rmf conftest.c conftest.o conftest a.out* a.exe* vv.out
+rm -f scn.c scn.o scn a.out* a.exe* vv.out
 
 #
 # Compiler: works as-is, with -Wno-error and -Werror
@@ -863,7 +789,7 @@ test x"$i" = x"" && if test $ct = sunpro; then
 	EOF
 	yes pad | head -n 256 >>x
 	ac_flags - 1 otwo -xO2 <x
-	rmf x
+	rm -f x
 elif test $ct = hpcc; then
 	phase=u
 	ac_flags 1 otwo +O2
@@ -895,11 +821,8 @@ elif test $ct = icc; then
 	ac_flags 1 fstacksecuritycheck -fstack-security-check
 	i=1
 elif test $ct = sunpro; then
-	phase=u
 	ac_flags 1 v -v
 	ac_flags 1 xc99 -xc99 'for support of ISO C99'
-	ac_flags 1 ipo -xipo 'for cross-module optimisation'
-	phase=x
 elif test $ct = hpcc; then
 	phase=u
 	ac_flags 1 agcc -Agcc 'for support of GCC extensions'
@@ -922,7 +845,7 @@ elif test $ct = msc; then
 	ac_flags - 1 stackon "${ccpc}/GZ" 'if stack checks can be enabled' <x
 	ac_flags - 1 stckall "${ccpc}/Ge" 'stack checks for all functions' <x
 	ac_flags - 1 secuchk "${ccpc}/GS" 'for compiler security checks' <x
-	rmf x
+	rm -f x
 	ac_flags 1 wall "${ccpc}/Wall" 'to enable all warnings'
 	ac_flags 1 wp64 "${ccpc}/Wp64" 'to enable 64-bit warnings'
 elif test $ct = xlc; then
@@ -961,7 +884,7 @@ test $ct = pcc && phase=u
 #
 # Compiler: check for stuff that only generates warnings
 #
-ac_test attribute_bounded '' 'for __attribute__((__bounded__))' <<-'EOF'
+ac_test attribute_bounded '' 'for __attribute__((bounded))' <<-'EOF'
 	#if defined(__GNUC__) && (__GNUC__ < 2)
 	/* force a failure: gcc 1.42 has a false positive here */
 	int main(void) { return (thiswillneverbedefinedIhope()); }
@@ -969,69 +892,68 @@ ac_test attribute_bounded '' 'for __attribute__((__bounded__))' <<-'EOF'
 	#include <string.h>
 	#undef __attribute__
 	int xcopy(const void *, void *, size_t)
-	    __attribute__((__bounded__ (__buffer__, 1, 3)))
-	    __attribute__((__bounded__ (__buffer__, 2, 3)));
+	    __attribute__((bounded (buffer, 1, 3)))
+	    __attribute__((bounded (buffer, 2, 3)));
 	int main(int ac, char *av[]) { return (xcopy(av[0], av[--ac], 1)); }
 	int xcopy(const void *s, void *d, size_t n) {
 		memmove(d, s, n); return ((int)n);
 	}
 	#endif
 EOF
-ac_test attribute_format '' 'for __attribute__((__format__))' <<-'EOF'
+ac_test attribute_format '' 'for __attribute__((format))' <<-'EOF'
 	#if defined(__GNUC__) && (__GNUC__ < 2)
 	/* force a failure: gcc 1.42 has a false positive here */
 	int main(void) { return (thiswillneverbedefinedIhope()); }
 	#else
-	#define fprintf printfoo
 	#include <stdio.h>
 	#undef __attribute__
-	#undef fprintf
-	extern int fprintf(FILE *, const char *format, ...)
-	    __attribute__((__format__ (__printf__, 2, 3)));
-	int main(int ac, char **av) { return (fprintf(stderr, "%s%d", *av, ac)); }
+	#undef printf
+	extern int printf(const char *format, ...)
+	    __attribute__((format (printf, 1, 2)));
+	int main(int ac, char **av) { return (printf("%s%d", *av, ac)); }
 	#endif
 EOF
-ac_test attribute_nonnull '' 'for __attribute__((__nonnull__))' <<-'EOF'
+ac_test attribute_nonnull '' 'for __attribute__((nonnull))' <<-'EOF'
 	#if defined(__GNUC__) && (__GNUC__ < 2)
 	/* force a failure: gcc 1.42 has a false positive here */
 	int main(void) { return (thiswillneverbedefinedIhope()); }
 	#else
-	int foo(char *s1, char *s2) __attribute__((__nonnull__));
-	int bar(char *s1, char *s2) __attribute__((__nonnull__ (1, 2)));
-	int baz(char *s) __attribute__((__nonnull__ (1)));
+	int foo(char *s1, char *s2) __attribute__((nonnull));
+	int bar(char *s1, char *s2) __attribute__((nonnull (1, 2)));
+	int baz(char *s) __attribute__((nonnull (1)));
 	int foo(char *s1, char *s2) { return (bar(s2, s1)); }
 	int bar(char *s1, char *s2) { return (baz(s1) - baz(s2)); }
 	int baz(char *s) { return (*s); }
 	int main(int ac, char **av) { return (ac == foo(av[0], av[ac-1])); }
 	#endif
 EOF
-ac_test attribute_noreturn '' 'for __attribute__((__noreturn__))' <<-'EOF'
+ac_test attribute_noreturn '' 'for __attribute__((noreturn))' <<-'EOF'
 	#if defined(__GNUC__) && (__GNUC__ < 2)
 	/* force a failure: gcc 1.42 has a false positive here */
 	int main(void) { return (thiswillneverbedefinedIhope()); }
 	#else
 	#include <stdlib.h>
 	#undef __attribute__
-	void fnord(void) __attribute__((__noreturn__));
+	void fnord(void) __attribute__((noreturn));
 	int main(void) { fnord(); }
 	void fnord(void) { exit(0); }
 	#endif
 EOF
-ac_test attribute_unused '' 'for __attribute__((__unused__))' <<-'EOF'
+ac_test attribute_unused '' 'for __attribute__((unused))' <<-'EOF'
 	#if defined(__GNUC__) && (__GNUC__ < 2)
 	/* force a failure: gcc 1.42 has a false positive here */
 	int main(void) { return (thiswillneverbedefinedIhope()); }
 	#else
-	int main(int ac __attribute__((__unused__)), char **av
-	    __attribute__((__unused__))) { return (0); }
+	int main(int ac __attribute__((unused)), char **av
+	    __attribute__((unused))) { return (0); }
 	#endif
 EOF
-ac_test attribute_used '' 'for __attribute__((__used__))' <<-'EOF'
+ac_test attribute_used '' 'for __attribute__((used))' <<-'EOF'
 	#if defined(__GNUC__) && (__GNUC__ < 2)
 	/* force a failure: gcc 1.42 has a false positive here */
 	int main(void) { return (thiswillneverbedefinedIhope()); }
 	#else
-	static const char fnord[] __attribute__((__used__)) = "42";
+	static const char fnord[] __attribute__((used)) = "42";
 	int main(void) { return (0); }
 	#endif
 EOF
@@ -1063,47 +985,58 @@ if ac_ifcpp 'ifdef MKSH_SMALL' isset_MKSH_SMALL '' \
 		;;
 	esac
 
-	: ${HAVE_NICE=0}
+	: ${HAVE_MKNOD=0}
+	: ${HAVE_REVOKE=0}
 	: ${HAVE_PERSISTENT_HISTORY=0}
-	check_categories="$check_categories smksh"
+	check_categories=$check_categories,smksh
 	HAVE_ISSET_MKSH_CONSERVATIVE_FDS=1	# from sh.h
 fi
 ac_ifcpp 'ifdef MKSH_BINSHREDUCED' isset_MKSH_BINSHREDUCED '' \
     "if a reduced-feature sh is requested" && \
-    check_categories="$check_categories binsh"
+    check_categories=$check_categories,binsh
 ac_ifcpp 'ifdef MKSH_UNEMPLOYED' isset_MKSH_UNEMPLOYED '' \
     "if mksh will be built without job control" && \
-    check_categories="$check_categories arge"
-ac_ifcpp 'ifdef MKSH_NOPROSPECTOFWORK' isset_MKSH_NOPROSPECTOFWORK '' \
-    "if mksh will be built without job signals" && \
-    check_categories="$check_categories arge nojsig"
+    check_categories=$check_categories,arge
 ac_ifcpp 'ifdef MKSH_ASSUME_UTF8' isset_MKSH_ASSUME_UTF8 '' \
     'if the default UTF-8 mode is specified' && : ${HAVE_SETLOCALE_CTYPE=0}
 ac_ifcpp 'ifdef MKSH_CONSERVATIVE_FDS' isset_MKSH_CONSERVATIVE_FDS '' \
     'if traditional/conservative fd use is requested' && \
-    check_categories="$check_categories convfds"
+    check_categories=$check_categories,convfds
 
 #
 # Environment: headers
 #
-ac_header sys/bsdtypes.h
-ac_header sys/file.h sys/types.h
+ac_header sys/param.h
 ac_header sys/mkdev.h sys/types.h
 ac_header sys/mman.h sys/types.h
-ac_header sys/param.h
-ac_header sys/select.h sys/types.h
 ac_header sys/sysmacros.h
-ac_header bstring.h
-ac_header grp.h sys/types.h
 ac_header libgen.h
 ac_header libutil.h sys/types.h
 ac_header paths.h
-ac_header stdint.h stdarg.h
-# include strings.h only if compatible with string.h
-ac_header strings.h sys/types.h string.h
+ac_header stdbool.h
+ac_header strings.h sys/types.h
+ac_header grp.h sys/types.h
 ac_header ulimit.h sys/types.h
 ac_header values.h
 
+ac_header '!' stdint.h stdarg.h
+ac_testn can_inttypes '!' stdint_h 1 "for standard 32-bit integer types" <<-'EOF'
+	#include <sys/types.h>
+	int main(int ac, char **av) { return ((uint32_t)*av + (int32_t)ac); }
+EOF
+ac_testn can_ucbints '!' can_inttypes 1 "for UCB 32-bit integer types" <<-'EOF'
+	#include <sys/types.h>
+	int main(int ac, char **av) { return ((u_int32_t)*av + (int32_t)ac); }
+EOF
+case $HAVE_CAN_INTTYPES$HAVE_CAN_UCBINTS in
+01)	HAVE_U_INT32_T=1
+	echo 'typedef u_int32_t uint32_t;' >>stdint.h ;;
+00)	echo 'typedef signed int int32_t;' >>stdint.h
+	echo 'typedef unsigned int uint32_t;' >>stdint.h ;;
+esac
+test -f stdint.h && HAVE_STDINT_H=1
+ac_cppflags STDINT_H
+
 #
 # Environment: definitions
 #
@@ -1122,32 +1055,11 @@ if test 0 = $HAVE_CAN_LFS_SUS; then
 	ac_testn can_lfs_aix '!' can_lfs 0 "... with -D_LARGE_FILES=1" <lft.c
 	test 1 = $HAVE_CAN_LFS_AIX || CPPFLAGS=$save_CPPFLAGS
 fi
-rmf lft*	# end of large file support test
+rm -f lft.c lft.o	# end of large file support test
 
 #
 # Environment: types
 #
-ac_test can_inttypes '!' stdint_h 1 "for standard 32-bit integer types" <<-'EOF'
-	#include <sys/types.h>
-	#include <stddef.h>
-	int main(int ac, char **av) { return ((uint32_t)(ptrdiff_t)*av + (int32_t)ac); }
-EOF
-ac_test can_ucbints '!' can_inttypes 1 "for UCB 32-bit integer types" <<-'EOF'
-	#include <sys/types.h>
-	#include <stddef.h>
-	int main(int ac, char **av) { return ((u_int32_t)(ptrdiff_t)*av + (int32_t)ac); }
-EOF
-ac_test can_int8type '!' stdint_h 1 "for standard 8-bit integer type" <<-'EOF'
-	#include <sys/types.h>
-	#include <stddef.h>
-	int main(int ac, char **av) { return ((uint8_t)(ptrdiff_t)av[ac]); }
-EOF
-ac_test can_ucbint8 '!' can_int8type 1 "for UCB 8-bit integer type" <<-'EOF'
-	#include <sys/types.h>
-	#include <stddef.h>
-	int main(int ac, char **av) { return ((u_int8_t)(ptrdiff_t)av[ac]); }
-EOF
-
 ac_test rlim_t <<-'EOF'
 	#include <sys/types.h>
 	#include <sys/time.h>
@@ -1161,14 +1073,14 @@ ac_testn sig_t <<-'EOF'
 	#include <sys/types.h>
 	#include <signal.h>
 	#include <stddef.h>
-	int main(void) { return ((int)(ptrdiff_t)(sig_t)(ptrdiff_t)kill(0,0)); }
+	int main(void) { return ((int)(ptrdiff_t)(sig_t)kill(0,0)); }
 EOF
 
 ac_testn sighandler_t '!' sig_t 0 <<-'EOF'
 	#include <sys/types.h>
 	#include <signal.h>
 	#include <stddef.h>
-	int main(void) { return ((int)(ptrdiff_t)(sighandler_t)(ptrdiff_t)kill(0,0)); }
+	int main(void) { return ((int)(ptrdiff_t)(sighandler_t)kill(0,0)); }
 EOF
 if test 1 = $HAVE_SIGHANDLER_T; then
 	CPPFLAGS="$CPPFLAGS -Dsig_t=sighandler_t"
@@ -1179,7 +1091,7 @@ ac_testn __sighandler_t '!' sig_t 0 <<-'EOF'
 	#include <sys/types.h>
 	#include <signal.h>
 	#include <stddef.h>
-	int main(void) { return ((int)(ptrdiff_t)(__sighandler_t)(ptrdiff_t)kill(0,0)); }
+	int main(void) { return ((int)(ptrdiff_t)(__sighandler_t)kill(0,0)); }
 EOF
 if test 1 = $HAVE___SIGHANDLER_T; then
 	CPPFLAGS="$CPPFLAGS -Dsig_t=__sighandler_t"
@@ -1189,6 +1101,15 @@ fi
 test 1 = $HAVE_SIG_T || CPPFLAGS="$CPPFLAGS -Dsig_t=nosig_t"
 ac_cppflags SIG_T
 
+ac_testn u_int32_t <<-'EOF'
+	#include <sys/types.h>
+	#if HAVE_STDINT_H
+	#include <stdint.h>
+	#endif
+	int main(void) { return ((int)(u_int32_t)0); }
+EOF
+test 1 = $HAVE_U_INT32_T || CPPFLAGS="$CPPFLAGS -Du_int32_t=uint32_t"
+
 #
 # Environment: signals
 #
@@ -1220,14 +1141,55 @@ EOF
 #
 # Environment: library functions
 #
+ac_testn arc4random <<-'EOF'
+	#include <sys/types.h>
+	#if HAVE_STDINT_H
+	#include <stdint.h>
+	#endif
+	extern u_int32_t arc4random(void);
+	int main(void) { return ((int)(arc4random() & 0xFF)); }
+EOF
+
+save_LIBS=$LIBS
+if test 0 = $HAVE_ARC4RANDOM; then
+	test -f arc4random.c || if test -f "$srcdir/arc4random.c"; then
+		# ensure isolation of source directory from build directory
+		cp "$srcdir/arc4random.c" .
+	fi
+	if test -f arc4random.c; then
+		ac_testn can_uint8t '' "for uint8_t" <<-'EOF'
+			#include <sys/types.h>
+			#if HAVE_STDINT_H
+			#include <stdint.h>
+			#endif
+			int main(void) { return (1 - (uint8_t)1); }
+		EOF
+		test $HAVE_CAN_UINT8T = 1 || \
+		    CPPFLAGS="$CPPFLAGS -D\"uint8_t=unsigned char\""
+
+		ac_header sys/sysctl.h
+		addsrcs '!' HAVE_ARC4RANDOM arc4random.c
+		HAVE_ARC4RANDOM=1
+		LIBS="$LIBS arc4random.c"
+	fi
+fi
+ac_cppflags ARC4RANDOM
+
+ac_test arc4random_pushb arc4random 0 <<-'EOF'
+	#include <sys/types.h>
+	#if HAVE_STDINT_H
+	#include <stdint.h>
+	#endif
+	extern uint32_t arc4random_pushb(void *, size_t);
+	int main(int ac, char *av[]) { return ((int)(arc4random_pushb(*av,
+	    (size_t)ac)) & 0xFF); }
+EOF
+LIBS=$save_LIBS
+
 ac_testn flock_ex '' 'flock and mmap' <<-'EOF'
 	#include <sys/types.h>
-	#if HAVE_SYS_FILE_H
 	#include <sys/file.h>
-	#endif
-	#if HAVE_SYS_MMAN_H
 	#include <sys/mman.h>
-	#endif
 	#include <fcntl.h>
 	#include <stdlib.h>
 	int main(void) { return ((void *)mmap(NULL, (size_t)flock(0, LOCK_EX),
@@ -1293,30 +1255,18 @@ ac_test langinfo_codeset setlocale_ctype 0 'nl_langinfo(CODESET)' <<-'EOF'
 	int main(void) { return ((int)(ptrdiff_t)(void *)nl_langinfo(CODESET)); }
 EOF
 
-ac_test select <<-'EOF'
+ac_test setmode mknod 1 <<-'EOF'
+	/* XXX imake style */
+	/* XXX conditions correct? */
+	#if defined(__MSVCRT__) || defined(__CYGWIN__)
+	/* force a failure: Win32 setmode() is not what we want... */
+	int main(void) { return (thiswillneverbedefinedIhope()); }
+	#else
 	#include <sys/types.h>
-	#include <sys/time.h>
-	#if HAVE_SYS_BSDTYPES_H
-	#include <sys/bsdtypes.h>
-	#endif
-	#if HAVE_SYS_SELECT_H
-	#include <sys/select.h>
-	#endif
-	#if HAVE_BSTRING_H
-	#include <bstring.h>
-	#endif
-	#include <stddef.h>
-	#include <stdlib.h>
-	#include <string.h>
-	#if HAVE_STRINGS_H
-	#include <strings.h>
-	#endif
 	#include <unistd.h>
-	int main(void) {
-		struct timeval tv = { 1, 200000 };
-		fd_set fds; FD_ZERO(&fds); FD_SET(0, &fds);
-		return (select(FD_SETSIZE, &fds, NULL, NULL, &tv));
-	}
+	int main(int ac, char *av[]) { return (getmode(setmode(av[0]),
+	    (mode_t)ac)); }
+	#endif
 EOF
 
 ac_test setresugid <<-'EOF'
@@ -1357,6 +1307,18 @@ EOF
 #
 save_CC=$CC; save_LDFLAGS=$LDFLAGS; save_LIBS=$LIBS
 CC="$CC -c -o $tcfn"; LDFLAGS=; LIBS=
+ac_test '!' arc4random_decl arc4random 1 'if arc4random() does not need to be declared' <<-'EOF'
+	#define MKSH_INCLUDES_ONLY
+	#include "sh.h"
+	long arc4random(void);		/* this clashes if defined before */
+	int main(void) { return ((int)arc4random()); }
+EOF
+ac_test '!' arc4random_pushb_decl arc4random_pushb 1 'if arc4random_pushb() does not need to be declared' <<-'EOF'
+	#define MKSH_INCLUDES_ONLY
+	#include "sh.h"
+	int arc4random_pushb(char, int); /* this clashes if defined before */
+	int main(int ac, char *av[]) { return ((int)arc4random_pushb(**av, ac)); }
+EOF
 ac_test '!' flock_decl flock_ex 1 'if flock() does not need to be declared' <<-'EOF'
 	#define MKSH_INCLUDES_ONLY
 	#include "sh.h"
@@ -1381,7 +1343,7 @@ CC=$save_CC; LDFLAGS=$save_LDFLAGS; LIBS=$save_LIBS
 #
 fd='if to use persistent history'
 ac_cache PERSISTENT_HISTORY || test 0 = $HAVE_FLOCK_EX || fv=1
-test 1 = $fv || check_categories="$check_categories no-histfile"
+test 1 = $fv || check_categories=$check_categories,no-histfile
 ac_testdone
 ac_cppflags
 
@@ -1390,10 +1352,10 @@ ac_cppflags
 #
 test 0 = $HAVE_SYS_SIGNAME && if ac_testinit cpp_dd '' \
     'checking if the C Preprocessor supports -dD'; then
-	echo '#define foo bar' >conftest.c
-	vv ']' "$CPP $CFLAGS $CPPFLAGS $NOWARN -dD conftest.c >x"
+	echo '#define foo bar' >scn.c
+	vv ']' "$CPP $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -dD scn.c $LIBS >x"
 	grep '#define foo bar' x >/dev/null 2>&1 && fv=1
-	rmf conftest.c x vv.out
+	rm -f scn.c x vv.out
 	ac_testdone
 fi
 
@@ -1406,8 +1368,8 @@ $e ... done.
 # the character count to standard output; cope for that
 echo wq >x
 ed x <x 2>/dev/null | grep 3 >/dev/null 2>&1 && \
-    check_categories="$check_categories $oldish_ed"
-rmf x vv.out
+    check_categories=$check_categories,$oldish_ed
+rm -f x vv.out
 
 if test 0 = $HAVE_SYS_SIGNAME; then
 	if test 1 = $HAVE_CPP_DD; then
@@ -1424,8 +1386,8 @@ if test 0 = $HAVE_SYS_SIGNAME; then
 #define NSIG (SIGMAX+1)
 #endif
 #endif
-mksh_cfg: NSIG' >conftest.c
-	NSIG=`vq "$CPP $CFLAGS $CPPFLAGS $NOWARN conftest.c" | \
+mksh_cfg: NSIG' >scn.c
+	NSIG=`vq "$CPP $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN scn.c $LIBS" | \
 	    grep mksh_cfg: | sed 's/^mksh_cfg:[	 ]*\([0-9x ()+-]*\).*$/\1/'`
 	case $NSIG in
 	*[\ \(\)+-]*) NSIG=`awk "BEGIN { print $NSIG }"` ;;
@@ -1438,14 +1400,14 @@ mksh_cfg: NSIG' >conftest.c
 	sigs="$sigs KILL LOST PIPE PROF PWR QUIT RESV SAK SEGV STOP SYS TERM"
 	sigs="$sigs TRAP TSTP TTIN TTOU URG USR1 USR2 VTALRM WINCH XCPU XFSZ"
 	test 1 = $HAVE_CPP_DD && test $NSIG -gt 1 && sigs="$sigs "`vq \
-	    "$CPP $CFLAGS $CPPFLAGS $NOWARN -dD conftest.c" | \
+	    "$CPP $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN -dD scn.c $LIBS" | \
 	    grep '[	 ]SIG[A-Z0-9]*[	 ]' | \
 	    sed 's/^\(.*[	 ]SIG\)\([A-Z0-9]*\)\([	 ].*\)$/\2/' | sort`
 	test $NSIG -gt 1 || sigs=
 	for name in $sigs; do
-		echo '#include <signal.h>' >conftest.c
-		echo mksh_cfg: SIG$name >>conftest.c
-		vq "$CPP $CFLAGS $CPPFLAGS $NOWARN conftest.c" | \
+		echo '#include <signal.h>' >scn.c
+		echo mksh_cfg: SIG$name >>scn.c
+		vq "$CPP $CFLAGS $CPPFLAGS $LDFLAGS $NOWARN scn.c $LIBS" | \
 		    grep mksh_cfg: | \
 		    sed 's/^mksh_cfg:[	 ]*\([0-9x]*\).*$/\1:'$name/
 	done | grep -v '^:' | while IFS=: read nr name; do
@@ -1459,17 +1421,17 @@ mksh_cfg: NSIG' >conftest.c
 			;;
 		esac
 	done 2>&1 >signames.inc
-	rmf conftest.c
+	rm -f scn.c
 	$e done.
 fi
 
+addsrcs '!' HAVE_SETMODE setmode.c
 addsrcs '!' HAVE_STRLCPY strlcpy.c
 addsrcs USE_PRINTF_BUILTIN printf.c
 test 1 = "$USE_PRINTF_BUILTIN" && CPPFLAGS="$CPPFLAGS -DMKSH_PRINTF_BUILTIN"
+test 0 = "$HAVE_SETMODE" && CPPFLAGS="$CPPFLAGS -DHAVE_CONFIG_H -DCONFIG_H_FILENAME=\\\"sh.h\\\""
 test 1 = "$HAVE_CAN_VERB" && CFLAGS="$CFLAGS -verbose"
 
-$e $bi$me: Finished configuration testing, now producing output.$ao
-
 files=
 objs=
 sp=
@@ -1480,42 +1442,7 @@ esac
 cat >>test.sh <<-EOF
 	LC_ALL=C PATH='$PATH'; export LC_ALL PATH
 	test -n "\$KSH_VERSION" || exit 1
-	set -A check_categories -- $check_categories
-	pflag='$curdir/mksh'
-	sflag='$srcdir/check.t'
-	usee=0 Pflag=0 uset=0 vflag=0 xflag=0
-	while getopts "C:e:Pp:s:t:v" ch; do case \$ch {
-	(C)	check_categories[\${#check_categories[*]}]=\$OPTARG ;;
-	(e)	usee=1; eflag=\$OPTARG ;;
-	(P)	Pflag=1 ;;
-	(p)	pflag=\$OPTARG ;;
-	(s)	sflag=\$OPTARG ;;
-	(t)	uset=1; tflag=\$OPTARG ;;
-	(v)	vflag=1 ;;
-	(*)	xflag=1 ;;
-	}
-	done
-	shift \$((OPTIND - 1))
-	set -A args -- '$srcdir/check.pl' -p "\$pflag" -s "\$sflag"
-	x=
-	for y in "\${check_categories[@]}"; do
-		x=\$x,\$y
-	done
-	if [[ -n \$x ]]; then
-		args[\${#args[*]}]=-C
-		args[\${#args[*]}]=\${x#,}
-	fi
-	if (( usee )); then
-		args[\${#args[*]}]=-e
-		args[\${#args[*]}]=\$eflag
-	fi
-	(( Pflag )) && args[\${#args[*]}]=-P
-	if (( uset )); then
-		args[\${#args[*]}]=-t
-		args[\${#args[*]}]=\$tflag
-	fi
-	(( vflag )) && args[\${#args[*]}]=-v
-	(( xflag )) && args[\${#args[*]}]=-x	# force usage by synerr
+	check_categories=$check_categories
 	print Testing mksh for conformance:
 	fgrep MirOS: '$srcdir/check.t'
 	fgrep MIRBSD '$srcdir/check.t'
@@ -1525,46 +1452,35 @@ cat >>test.sh <<-EOF
 	cstr="\$cstr"'print \$os . ", Perl version " . \$];'
 	for perli in \$PERL perl5 perl no; do
 		[[ \$perli = no ]] && exit 1
-		perlos=\$(\$perli -e "\$cstr") 2>/dev/null || continue
+		perlos=\$(\$perli -e "\$cstr") 2>&- || continue
 		print "Perl interpreter '\$perli' running on '\$perlos'"
 		[[ -n \$perlos ]] && break
 	done
-	exec \$perli "\${args[@]}" "\$@"$tsts
+	exec \$perli '$srcdir/check.pl' -s '$srcdir/check.t' -p '$curdir/mksh' \${check_categories:+-C} \${check_categories#,} \$*$tsts
 EOF
 chmod 755 test.sh
 test $HAVE_CAN_COMBINE$cm = 0combine && cm=normal
 if test $cm = llvm; then
 	emitbc="-emit-llvm -c"
-elif test $cm = dragonegg; then
-	emitbc="-S -flto"
 else
 	emitbc=-c
 fi
 echo set -x >Rebuild.sh
 for file in $SRCS; do
-	op=`echo x"$file" | sed 's/^x\(.*\)\.c$/\1./'`
+	of=`echo x"$file" | sed 's/^x\(.*\)\.c$/\1.o/'`
+	objs="$objs$sp$of"
 	test -f $file || file=$srcdir/$file
 	files="$files$sp$file"
 	sp=' '
 	echo "$CC $CFLAGS $CPPFLAGS $emitbc $file || exit 1" >>Rebuild.sh
-	if test $cm = dragonegg; then
-		echo "mv ${op}s ${op}ll" >>Rebuild.sh
-		echo "llvm-as ${op}ll || exit 1" >>Rebuild.sh
-		objs="$objs$sp${op}bc"
-	else
-		objs="$objs$sp${op}o"
-	fi
 done
-case $cm in
-dragonegg|llvm)
+if test $cm = llvm; then
 	echo "rm -f mksh.s" >>Rebuild.sh
-	echo "llvm-link -o - $objs | opt $optflags | llc -o mksh.s" >>Rebuild.sh
+	echo "llvm-link -o - $objs | opt $llvm | llc -o mksh.s" >>Rebuild.sh
 	lobjs=mksh.s
-	;;
-*)
+else
 	lobjs=$objs
-	;;
-esac
+fi
 case $tcfn in
 a.exe)	mkshexe=mksh.exe ;;
 *)	mkshexe=mksh ;;
@@ -1630,27 +1546,19 @@ elif test 1 = $pm; then
 	wait
 else
 	for file in $SRCS; do
-		test $cm = dragonegg && \
-		    op=`echo x"$file" | sed 's/^x\(.*\)\.c$/\1./'`
 		test -f $file || file=$srcdir/$file
 		v "$CC $CFLAGS $CPPFLAGS $emitbc $file" || exit 1
-		if test $cm = dragonegg; then
-			v "mv ${op}s ${op}ll"
-			v "llvm-as ${op}ll" || exit 1
-		fi
 	done
 fi
-case $cm in
-dragonegg|llvm)
-	rmf mksh.s
-	v "llvm-link -o - $objs | opt $optflags | llc -o mksh.s"
-	;;
-esac
+if test $cm = llvm; then
+	rm -f mksh.s
+	v "llvm-link -o - $objs | opt $llvm | llc -o mksh.s"
+fi
 tcfn=$mkshexe
 test $cm = combine || v "$CC $CFLAGS $LDFLAGS -o $tcfn $lobjs $LIBS $ccpr"
 test -f $tcfn || exit 1
 test 1 = $r || v "$NROFF -mdoc <'$srcdir/mksh.1' >mksh.cat1" || \
-    rmf mksh.cat1
+    rm -f mksh.cat1
 test 0 = $eq && v size $tcfn
 i=install
 test -f /usr/ucb/$i && i=/usr/ucb/$i
