@@ -1,8 +1,8 @@
-/*	$OpenBSD: history.c,v 1.39 2010/05/19 17:36:08 jasper Exp $	*/
-/*	$OpenBSD: trap.c,v 1.23 2010/05/19 17:36:08 jasper Exp $	*/
+/*	$OpenBSD: history.c,v 1.37 2009/07/02 16:29:15 martynas Exp $	*/
+/*	$OpenBSD: trap.c,v 1.22 2005/03/30 17:16:37 deraadt Exp $	*/
 
 /*-
- * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+ * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009
  *	Thorsten Glaser <tg@mirbsd.org>
  *
  * Provided that these terms and disclaimer and all copyright notices
@@ -22,11 +22,11 @@
  */
 
 #include "sh.h"
-#if HAVE_SYS_FILE_H
+#if HAVE_PERSISTENT_HISTORY
 #include <sys/file.h>
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.108 2011/03/07 20:30:38 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.90 2009/12/12 22:27:08 tg Exp $");
 
 /*-
  * MirOS: This is the default mapping type, and need not be specified.
@@ -50,8 +50,8 @@ static int sprinkle(int);
 #endif
 
 static int hist_execute(char *);
-static int hist_replace(char **, const char *, const char *, bool);
-static char **hist_get(const char *, bool, bool);
+static int hist_replace(char **, const char *, const char *, int);
+static char **hist_get(const char *, int, int);
 static char **hist_get_oldest(void);
 static void histbackup(void);
 
@@ -60,15 +60,11 @@ static int hstarted;		/* set after hist_init() called */
 static Source *hist_source;
 
 #if HAVE_PERSISTENT_HISTORY
-/* current history file: name, fd, size */
-static char *hname;
+static char *hname;		/* current name of history file */
 static int histfd;
 static int hsize;
 #endif
 
-static const char T_not_in_history[] = "not in history";
-#define T_history (T_not_in_history + 7)
-
 int
 c_fc(const char **wp)
 {
@@ -76,57 +72,45 @@ c_fc(const char **wp)
 	struct temp *tf;
 	const char *p;
 	char *editor = NULL;
-	bool gflag = false, lflag = false, nflag = false, rflag = false,
-	    sflag = false;
+	int gflag = 0, lflag = 0, nflag = 0, sflag = 0, rflag = 0;
 	int optc;
 	const char *first = NULL, *last = NULL;
 	char **hfirst, **hlast, **hp;
 
 	if (!Flag(FTALKING_I)) {
-		bi_errorf("history %ss not available", T_function);
+		bi_errorf("history functions not available");
 		return (1);
 	}
 
 	while ((optc = ksh_getopt(wp, &builtin_opt,
 	    "e:glnrs0,1,2,3,4,5,6,7,8,9,")) != -1)
 		switch (optc) {
-
 		case 'e':
 			p = builtin_opt.optarg;
 			if (ksh_isdash(p))
-				sflag = true;
+				sflag++;
 			else {
 				size_t len = strlen(p);
-
-				/* almost certainly not overflowing */
 				editor = alloc(len + 4, ATEMP);
 				memcpy(editor, p, len);
 				memcpy(editor + len, " $_", 4);
 			}
 			break;
-
-		/* non-AT&T ksh */
-		case 'g':
-			gflag = true;
+		case 'g': /* non-AT&T ksh */
+			gflag++;
 			break;
-
 		case 'l':
-			lflag = true;
+			lflag++;
 			break;
-
 		case 'n':
-			nflag = true;
+			nflag++;
 			break;
-
 		case 'r':
-			rflag = true;
+			rflag++;
 			break;
-
-		/* POSIX version of -e - */
-		case 's':
-			sflag = true;
+		case 's':	/* POSIX version of -e - */
+			sflag++;
 			break;
-
 		/* kludge city - accept -num as -- -num (kind of) */
 		case '0': case '1': case '2': case '3': case '4':
 		case '5': case '6': case '7': case '8': case '9':
@@ -141,7 +125,6 @@ c_fc(const char **wp)
 				return (1);
 			}
 			break;
-
 		case '?':
 			return (1);
 		}
@@ -199,15 +182,15 @@ c_fc(const char **wp)
 		/* can't fail if hfirst didn't fail */
 		hlast = hist_get_newest(false);
 	} else {
-		/*
-		 * POSIX says not an error if first/last out of bounds
-		 * when range is specified; AT&T ksh and pdksh allow out
-		 * of bounds for -l as well.
+		/* POSIX says not an error if first/last out of bounds
+		 * when range is specified; AT&T ksh and pdksh allow out of
+		 * bounds for -l as well.
 		 */
-		hfirst = hist_get(first, tobool(lflag || last), lflag);
+		hfirst = hist_get(first, (lflag || last) ? true : false,
+		    lflag ? true : false);
 		if (!hfirst)
 			return (1);
-		hlast = last ? hist_get(last, true, lflag) :
+		hlast = last ? hist_get(last, true, lflag ? true : false) :
 		    (lflag ? hist_get_newest(false) : hfirst);
 		if (!hlast)
 			return (1);
@@ -216,8 +199,7 @@ c_fc(const char **wp)
 		char **temp;
 
 		temp = hfirst; hfirst = hlast; hlast = temp;
-		/* POSIX */
-		rflag = !rflag;
+		rflag = !rflag; /* POSIX */
 	}
 
 	/* List history */
@@ -248,16 +230,15 @@ c_fc(const char **wp)
 
 	tf = maketemp(ATEMP, TT_HIST_EDIT, &e->temps);
 	if (!(shf = tf->shf)) {
-		bi_errorf("can't %s temporary file %s: %s",
-		    "create", tf->name, strerror(errno));
+		bi_errorf("cannot create temp file %s - %s",
+		    tf->name, strerror(errno));
 		return (1);
 	}
 	for (hp = rflag ? hlast : hfirst;
 	    hp >= hfirst && hp <= hlast; hp += rflag ? -1 : 1)
 		shf_fprintf(shf, "%s\n", *hp);
 	if (shf_close(shf) == EOF) {
-		bi_errorf("can't %s temporary file %s: %s",
-		    "write", tf->name, strerror(errno));
+		bi_errorf("error writing temporary file - %s", strerror(errno));
 		return (1);
 	}
 
@@ -269,7 +250,7 @@ c_fc(const char **wp)
 		Source *sold = source;
 		int ret;
 
-		ret = command(editor ? editor : "${FCEDIT:-/bin/ed} $_", 0);
+		ret = command(editor ? editor : "${FCEDIT:-/bin/ed} $_");
 		source = sold;
 		if (ret)
 			return (ret);
@@ -282,19 +263,11 @@ c_fc(const char **wp)
 		int n;
 
 		if (!(shf = shf_open(tf->name, O_RDONLY, 0, 0))) {
-			bi_errorf("can't %s temporary file %s: %s",
-			    "open", tf->name, strerror(errno));
+			bi_errorf("cannot open temp file %s", tf->name);
 			return (1);
 		}
 
-		if (stat(tf->name, &statb) < 0)
-			n = 128;
-		else if (statb.st_size > (1024 * 1048576)) {
-			bi_errorf("%s %s too large: %lu", T_history,
-			    "file", (unsigned long)statb.st_size);
-			goto errout;
-		} else
-			n = statb.st_size + 1;
+		n = stat(tf->name, &statb) < 0 ? 128 : statb.st_size + 1;
 		Xinit(xs, xp, n, hist_source->areap);
 		while ((n = shf_read(xp, Xnleft(xs, xp), shf)) > 0) {
 			xp += n;
@@ -302,9 +275,8 @@ c_fc(const char **wp)
 				XcheckN(xs, xp, Xlength(xs, xp));
 		}
 		if (n < 0) {
-			bi_errorf("can't %s temporary file %s: %s",
-			    "read", tf->name, strerror(shf_errno(shf)));
- errout:
+			bi_errorf("error reading temp file %s - %s",
+			    tf->name, strerror(shf_errno(shf)));
 			shf_close(shf);
 			return (1);
 		}
@@ -327,23 +299,18 @@ hist_execute(char *cmd)
 
 	for (p = cmd; p; p = q) {
 		if ((q = strchr(p, '\n'))) {
-			/* kill the newline */
-			*q++ = '\0';
-			if (!*q)
-				/* ignore trailing newline */
+			*q++ = '\0'; /* kill the newline */
+			if (!*q) /* ignore trailing newline */
 				q = NULL;
 		}
 		histsave(&hist_source->line, p, true, true);
 
-		/* POSIX doesn't say this is done... */
-		shellf("%s\n", p);
-		if (q)
-			/* restore \n (trailing \n not restored) */
+		shellf("%s\n", p); /* POSIX doesn't say this is done... */
+		if (q)		/* restore \n (trailing \n not restored) */
 			q[-1] = '\n';
 	}
 
-	/*-
-	 * Commands are executed here instead of pushing them onto the
+	/* Commands are executed here instead of pushing them onto the
 	 * input 'cause POSIX says the redirection and variable assignments
 	 * in
 	 *	X=y fc -e - 42 2> /dev/null
@@ -351,13 +318,13 @@ hist_execute(char *cmd)
 	 */
 	/* XXX: source should not get trashed by this.. */
 	sold = source;
-	ret = command(cmd, 0);
+	ret = command(cmd);
 	source = sold;
 	return (ret);
 }
 
 static int
-hist_replace(char **hp, const char *pat, const char *rep, bool globr)
+hist_replace(char **hp, const char *pat, const char *rep, int globr)
 {
 	char *line;
 
@@ -370,23 +337,21 @@ hist_replace(char **hp, const char *pat, const char *rep, bool globr)
 		int len;
 		XString xs;
 		char *xp;
-		bool any_subst = false;
+		int any_subst = 0;
 
 		Xinit(xs, xp, 128, ATEMP);
 		for (s = *hp; (s1 = strstr(s, pat)) && (!any_subst || globr);
 		    s = s1 + pat_len) {
-			any_subst = true;
+			any_subst = 1;
 			len = s1 - s;
 			XcheckN(xs, xp, len + rep_len);
-			/*; first part */
-			memcpy(xp, s, len);
+			memcpy(xp, s, len);		/* first part */
 			xp += len;
-			/* replacement */
-			memcpy(xp, rep, rep_len);
+			memcpy(xp, rep, rep_len);	/* replacement */
 			xp += rep_len;
 		}
 		if (!any_subst) {
-			bi_errorf("bad substitution");
+			bi_errorf("substitution failed");
 			return (1);
 		}
 		len = strlen(s) + 1;
@@ -403,7 +368,7 @@ hist_replace(char **hp, const char *pat, const char *rep, bool globr)
  * pattern is a number or string
  */
 static char **
-hist_get(const char *str, bool approx, bool allow_cur)
+hist_get(const char *str, int approx, int allow_cur)
 {
 	char **hp = NULL;
 	int n;
@@ -414,18 +379,18 @@ hist_get(const char *str, bool approx, bool allow_cur)
 			if (approx)
 				hp = hist_get_oldest();
 			else {
-				bi_errorf("%s: %s", str, T_not_in_history);
+				bi_errorf("%s: not in history", str);
 				hp = NULL;
 			}
 		} else if ((ptrdiff_t)hp > (ptrdiff_t)histptr) {
 			if (approx)
 				hp = hist_get_newest(allow_cur);
 			else {
-				bi_errorf("%s: %s", str, T_not_in_history);
+				bi_errorf("%s: not in history", str);
 				hp = NULL;
 			}
 		} else if (!allow_cur && hp == histptr) {
-			bi_errorf("%s: %s", str, "invalid range");
+			bi_errorf("%s: invalid range", str);
 			hp = NULL;
 		}
 	} else {
@@ -433,7 +398,7 @@ hist_get(const char *str, bool approx, bool allow_cur)
 
 		/* the -1 is to avoid the current fc command */
 		if ((n = findhist(histptr - history - 1, 0, str, anchored)) < 0)
-			bi_errorf("%s: %s", str, T_not_in_history);
+			bi_errorf("%s: not in history", str);
 		else
 			hp = &history[n];
 	}
@@ -442,7 +407,7 @@ hist_get(const char *str, bool approx, bool allow_cur)
 
 /* Return a pointer to the newest command in the history */
 char **
-hist_get_newest(bool allow_cur)
+hist_get_newest(int allow_cur)
 {
 	if (histptr < history || (!allow_cur && histptr == history)) {
 		bi_errorf("no history (yet)");
@@ -451,7 +416,7 @@ hist_get_newest(bool allow_cur)
 	return (allow_cur ? histptr : histptr - 1);
 }
 
-/* Return a pointer to the oldest command in the history */
+/* Return a pointer to the newest command in the history */
 static char **
 hist_get_oldest(void)
 {
@@ -462,9 +427,9 @@ hist_get_oldest(void)
 	return (history);
 }
 
-/*
- * Back up over last histsave
- */
+/******************************/
+/* Back up over last histsave */
+/******************************/
 static void
 histbackup(void)
 {
@@ -562,7 +527,7 @@ sethistsize(int n)
 			cursize = n;
 		}
 
-		history = aresize2(history, n, sizeof(char *), APERM);
+		history = aresize(history, n * sizeof(char *), APERM);
 
 		histsize = n;
 		histptr = history + cursize;
@@ -613,7 +578,7 @@ init_histvec(void)
 {
 	if (history == (char **)NULL) {
 		histsize = HISTORYSIZE;
-		history = alloc2(histsize, sizeof(char *), APERM);
+		history = alloc(histsize * sizeof(char *), APERM);
 		histptr = history - 1;
 	}
 }
@@ -627,27 +592,6 @@ init_histvec(void)
  *	It turns out that there is a lot of ghastly hackery here
  */
 
-#if !defined(MKSH_SMALL) && HAVE_PERSISTENT_HISTORY
-/* do not save command in history but possibly sync */
-bool
-histsync(void)
-{
-	bool changed = false;
-
-	if (histfd) {
-		int lno = hist_source->line;
-
-		hist_source->line++;
-		writehistfile(0, NULL);
-		hist_source->line--;
-
-		if (lno != hist_source->line)
-			changed = true;
-	}
-
-	return (changed);
-}
-#endif
 
 /*
  * save command in history
@@ -662,11 +606,7 @@ histsave(int *lnp, const char *cmd, bool dowrite MKSH_A_UNUSED, bool ignoredups)
 	if ((cp = strchr(c, '\n')) != NULL)
 		*cp = '\0';
 
-	if (ignoredups && !strcmp(c, *histptr)
-#if !defined(MKSH_SMALL) && HAVE_PERSISTENT_HISTORY
-	    && !histsync()
-#endif
-	    ) {
+	if (ignoredups && !strcmp(c, *histptr)) {
 		afree(c, APERM);
 		return;
 	}
@@ -679,8 +619,7 @@ histsave(int *lnp, const char *cmd, bool dowrite MKSH_A_UNUSED, bool ignoredups)
 
 	hp = histptr;
 
-	if (++hp >= history + histsize) {
-		/* remove oldest command */
+	if (++hp >= history + histsize) { /* remove oldest command */
 		afree(*history, APERM);
 		for (hp = history; hp < history + histsize - 1; hp++)
 			hp[0] = hp[1];
@@ -700,7 +639,7 @@ histsave(int *lnp, const char *cmd, bool dowrite MKSH_A_UNUSED, bool ignoredups)
  *	if your system ain't got it - then you'll have to undef HISTORYFILE
  */
 
-/*-
+/*
  *	Open a history file
  *	Format is:
  *	Bytes 1, 2:
@@ -781,9 +720,8 @@ hist_init(Source *s)
 				hist_finish();
 				if (rv) {
  hiniterr:
-					bi_errorf("can't %s %s: %s",
-					    "unlink HISTFILE", hname,
-					    strerror(errno));
+					bi_errorf("cannot unlink HISTFILE %s"
+					    " - %s", hname, strerror(errno));
 					hsize = 0;
 					return;
 				}
@@ -827,10 +765,8 @@ hist_count_lines(unsigned char *base, int bytes)
 		case sn4:
 			state = sline; break;
 		case sline:
-			if (*base == '\0') {
-				lines++;
-				state = shdr;
-			}
+			if (*base == '\0')
+				lines++, state = shdr;
 		}
 		base++;
 	}
@@ -897,10 +833,9 @@ hist_skip_back(unsigned char *base, int *bytes, int no)
 	unsigned char *ep;
 
 	for (ep = base + *bytes; --ep > base; ) {
-		/*
-		 * this doesn't really work: the 4 byte line number that
-		 * is encoded after the COMMAND byte can itself contain
-		 * the COMMAND byte....
+		/* this doesn't really work: the 4 byte line number that is
+		 * encoded after the COMMAND byte can itself contain the
+		 * COMMAND byte....
 		 */
 		for (; ep > base && *ep != COMMAND; ep--)
 			;
@@ -992,7 +927,7 @@ writehistfile(int lno, char *cmd)
 {
 	int	sizenow;
 	unsigned char	*base;
-	unsigned char	*news;
+	unsigned char	*new;
 	int	bytes;
 	unsigned char	hdr[5];
 
@@ -1009,13 +944,13 @@ writehistfile(int lno, char *cmd)
 			    MAP_FILE | MAP_PRIVATE, histfd, (off_t)0);
 			if (base == (unsigned char *)MAP_FAILED)
 				goto bad;
-			news = base + hsize;
-			if (*news != COMMAND) {
+			new = base + hsize;
+			if (*new != COMMAND) {
 				munmap((caddr_t)base, sizenow);
 				goto bad;
 			}
 			hist_source->line--;
-			histload(hist_source, news, bytes);
+			histload(hist_source, new, bytes);
 			hist_source->line++;
 			lno = hist_source->line;
 			munmap((caddr_t)base, sizenow);
@@ -1027,21 +962,19 @@ writehistfile(int lno, char *cmd)
 			goto bad;
 		}
 	}
-	if (cmd) {
-		/*
-		 *	we can write our bit now
-		 */
-		hdr[0] = COMMAND;
-		hdr[1] = (lno>>24)&0xff;
-		hdr[2] = (lno>>16)&0xff;
-		hdr[3] = (lno>>8)&0xff;
-		hdr[4] = lno&0xff;
-		bytes = strlen(cmd) + 1;
-		if ((write(histfd, hdr, 5) != 5) ||
-		    (write(histfd, cmd, bytes) != bytes))
-			goto bad;
-		hsize = lseek(histfd, (off_t)0, SEEK_END);
-	}
+	/*
+	 *	we can write our bit now
+	 */
+	hdr[0] = COMMAND;
+	hdr[1] = (lno>>24)&0xff;
+	hdr[2] = (lno>>16)&0xff;
+	hdr[3] = (lno>>8)&0xff;
+	hdr[4] = lno&0xff;
+	bytes = strlen(cmd) + 1;
+	if ((write(histfd, hdr, 5) != 5) ||
+	    (write(histfd, cmd, bytes) != bytes))
+		goto bad;
+	hsize = lseek(histfd, (off_t)0, SEEK_END);
 	(void)flock(histfd, LOCK_UN);
 	return;
  bad:
@@ -1084,8 +1017,6 @@ inittraps(void)
 	int i;
 	const char *cs;
 
-	trap_exstat = -1;
-
 	/* Populate sigtraps based on sys_signame and sys_siglist. */
 	for (i = 0; i <= NSIG; i++) {
 		sigtraps[i].signal = i;
@@ -1123,12 +1054,10 @@ inittraps(void)
 #endif
 			if ((sigtraps[i].mess == NULL) ||
 			    (sigtraps[i].mess[0] == '\0'))
-				sigtraps[i].mess = shf_smprintf("%s %d",
-				    "Signal", i);
+				sigtraps[i].mess = shf_smprintf("Signal %d", i);
 		}
 	}
-	/* our name for signal 0 */
-	sigtraps[SIGEXIT_].name = "EXIT";
+	sigtraps[SIGEXIT_].name = "EXIT";	/* our name for signal 0 */
 
 	(void)sigemptyset(&Sigact_ign.sa_mask);
 	Sigact_ign.sa_flags = 0; /* interruptible */
@@ -1136,8 +1065,7 @@ inittraps(void)
 
 	sigtraps[SIGINT].flags |= TF_DFL_INTR | TF_TTY_INTR;
 	sigtraps[SIGQUIT].flags |= TF_DFL_INTR | TF_TTY_INTR;
-	/* SIGTERM is not fatal for interactive */
-	sigtraps[SIGTERM].flags |= TF_DFL_INTR;
+	sigtraps[SIGTERM].flags |= TF_DFL_INTR;/* not fatal for interactive */
 	sigtraps[SIGHUP].flags |= TF_FATAL;
 	sigtraps[SIGCHLD].flags |= TF_SHELL_USES;
 
@@ -1162,7 +1090,7 @@ alarm_init(void)
 static void
 alarm_catcher(int sig MKSH_A_UNUSED)
 {
-	/* this runs inside interrupt context, with errno saved */
+	int errno_ = errno;
 
 	if (ksh_tmout_state == TMOUT_READING) {
 		int left = alarm(0);
@@ -1173,6 +1101,7 @@ alarm_catcher(int sig MKSH_A_UNUSED)
 		} else
 			alarm(left);
 	}
+	errno = errno_;
 }
 
 Trap *
@@ -1220,8 +1149,7 @@ trapsig(int i)
 	errno = errno_;
 }
 
-/*
- * called when we want to allow the user to ^C out of something - won't
+/* called when we want to allow the user to ^C out of something - won't
  * work if user has trapped SIGINT.
  */
 void
@@ -1231,8 +1159,7 @@ intrcheck(void)
 		runtraps(TF_DFL_INTR|TF_FATAL);
 }
 
-/*
- * called after EINTR to check if a signal with normally causes process
+/* called after EINTR to check if a signal with normally causes process
  * termination has been received.
  */
 int
@@ -1249,8 +1176,7 @@ fatal_trap_check(void)
 	return (0);
 }
 
-/*
- * Returns the signal number of any pending traps: ie, a signal which has
+/* Returns the signal number of any pending traps: ie, a signal which has
  * occurred for which a trap has been set or for which the TF_DFL_INTR flag
  * is set.
  */
@@ -1282,8 +1208,7 @@ runtraps(int flag)
 		warningf(false, "timed out waiting for input");
 		unwind(LEXIT);
 	} else
-		/*
-		 * XXX: this means the alarm will have no effect if a trap
+		/* XXX: this means the alarm will have no effect if a trap
 		 * is caught after the alarm() was started...not good.
 		 */
 		ksh_tmout_state = TMOUT_EXECUTING;
@@ -1293,29 +1218,22 @@ runtraps(int flag)
 		intrsig = 0;
 	if (flag & TF_FATAL)
 		fatal_trap = 0;
-	++trap_nested;
 	for (p = sigtraps, i = NSIG+1; --i >= 0; p++)
 		if (p->set && (!flag ||
 		    ((p->flags & flag) && p->trap == NULL)))
-			runtrap(p, false);
-	if (!--trap_nested)
-		runtrap(NULL, true);
+			runtrap(p);
 }
 
 void
-runtrap(Trap *p, bool is_last)
+runtrap(Trap *p)
 {
-	int old_changed = 0, i;
-	char *trapstr;
+	int	i = p->signal;
+	char	*trapstr = p->trap;
+	int	oexstat;
+	int	old_changed = 0;
 
-	if (p == NULL)
-		/* just clean up, see runtraps() above */
-		goto donetrap;
-	i = p->signal;
-	trapstr = p->trap;
 	p->set = 0;
-	if (trapstr == NULL) {
-		/* SIG_DFL */
+	if (trapstr == NULL) { /* SIG_DFL */
 		if (p->flags & TF_FATAL) {
 			/* eg, SIGHUP */
 			exstat = 128 + i;
@@ -1326,24 +1244,21 @@ runtrap(Trap *p, bool is_last)
 			exstat = 128 + i;
 			unwind(LINTR);
 		}
-		goto donetrap;
+		return;
 	}
-	if (trapstr[0] == '\0')
-		/* SIG_IGN */
-		goto donetrap;
-	if (i == SIGEXIT_ || i == SIGERR_) {
-		/* avoid recursion on these */
+	if (trapstr[0] == '\0') /* SIG_IGN */
+		return;
+	if (i == SIGEXIT_ || i == SIGERR_) {	/* avoid recursion on these */
 		old_changed = p->flags & TF_CHANGED;
 		p->flags &= ~TF_CHANGED;
 		p->trap = NULL;
 	}
-	if (trap_exstat == -1)
-		trap_exstat = exstat;
-	/*
-	 * Note: trapstr is fully parsed before anything is executed, thus
+	oexstat = exstat;
+	/* Note: trapstr is fully parsed before anything is executed, thus
 	 * no problem with afree(p->trap) in settrap() while still in use.
 	 */
-	command(trapstr, current_lineno);
+	command(trapstr);
+	exstat = oexstat;
 	if (i == SIGEXIT_ || i == SIGERR_) {
 		if (p->flags & TF_CHANGED)
 			/* don't clear TF_CHANGED */
@@ -1352,13 +1267,6 @@ runtrap(Trap *p, bool is_last)
 			p->trap = trapstr;
 		p->flags |= old_changed;
 	}
-
- donetrap:
-	/* we're the last trap of a sequence executed */
-	if (is_last && trap_exstat != -1) {
-		exstat = trap_exstat;
-		trap_exstat = -1;
-	}
 }
 
 /* clear pending traps and reset user's trap handlers; used after fork(2) */
@@ -1398,8 +1306,7 @@ settrap(Trap *p, const char *s)
 
 	if (p->trap)
 		afree(p->trap, APERM);
-	/* handles s == NULL */
-	strdupx(p->trap, s, APERM);
+	strdupx(p->trap, s, APERM); /* handles s == 0 */
 	p->flags |= TF_CHANGED;
 	f = !s ? SIG_DFL : s[0] ? trapsig : SIG_IGN;
 
@@ -1416,8 +1323,7 @@ settrap(Trap *p, const char *s)
 				p->flags |= TF_EXEC_DFL;
 		}
 
-		/*
-		 * assumes handler already set to what shell wants it
+		/* assumes handler already set to what shell wants it
 		 * (normally trapsig, but could be j_sigchld() or SIG_IGN)
 		 */
 		return;
@@ -1427,8 +1333,7 @@ settrap(Trap *p, const char *s)
 	setsig(p, f, SS_RESTORE_CURR|SS_USER);
 }
 
-/*
- * Called by c_print() when writing to a co-process to ensure SIGPIPE won't
+/* Called by c_print() when writing to a co-process to ensure SIGPIPE won't
  * kill shell (unless user catches it and exits)
  */
 int
@@ -1443,8 +1348,7 @@ block_pipe(void)
 			restore_dfl = 1;
 	} else if (p->cursig == SIG_DFL) {
 		setsig(p, SIG_IGN, SS_RESTORE_CURR);
-		/* restore to SIG_DFL */
-		restore_dfl = 1;
+		restore_dfl = 1; /* restore to SIG_DFL */
 	}
 	return (restore_dfl);
 }
@@ -1457,8 +1361,7 @@ restore_pipe(int restore_dfl)
 		setsig(&sigtraps[SIGPIPE], SIG_DFL, SS_RESTORE_CURR);
 }
 
-/*
- * Set action for a signal. Action may not be set if original
+/* Set action for a signal. Action may not be set if original
  * action was SIG_IGN, depending on the value of flags and FTALKING.
  */
 int
@@ -1469,8 +1372,7 @@ setsig(Trap *p, sig_t f, int flags)
 	if (p->signal == SIGEXIT_ || p->signal == SIGERR_)
 		return (1);
 
-	/*
-	 * First time setting this signal? If so, get and note the current
+	/* First time setting this signal? If so, get and note the current
 	 * setting.
 	 */
 	if (!(p->flags & (TF_ORIG_IGN|TF_ORIG_DFL))) {
@@ -1480,8 +1382,7 @@ setsig(Trap *p, sig_t f, int flags)
 		p->cursig = SIG_IGN;
 	}
 
-	/*-
-	 * Generally, an ignored signal stays ignored, except if
+	/* Generally, an ignored signal stays ignored, except if
 	 *	- the user of an interactive shell wants to change it
 	 *	- the shell wants for force a change
 	 */
@@ -1491,11 +1392,9 @@ setsig(Trap *p, sig_t f, int flags)
 
 	setexecsig(p, flags & SS_RESTORE_MASK);
 
-	/*
-	 * This is here 'cause there should be a way of clearing
-	 * shtraps, but don't know if this is a sane way of doing
-	 * it. At the moment, all users of shtrap are lifetime
-	 * users (SIGALRM, SIGCHLD, SIGWINCH).
+	/* This is here 'cause there should be a way of clearing shtraps, but
+	 * don't know if this is a sane way of doing it. At the moment,
+	 * all users of shtrap are lifetime users (SIGCHLD, SIGALRM, SIGWINCH).
 	 */
 	if (!(flags & SS_USER))
 		p->shtrap = (sig_t)NULL;
@@ -1507,8 +1406,7 @@ setsig(Trap *p, sig_t f, int flags)
 	if (p->cursig != f) {
 		p->cursig = f;
 		(void)sigemptyset(&sigact.sa_mask);
-		/* interruptible */
-		sigact.sa_flags = 0;
+		sigact.sa_flags = 0 /* interruptible */;
 		sigact.sa_handler = f;
 		sigaction(p->signal, &sigact, NULL);
 	}
@@ -1528,8 +1426,7 @@ setexecsig(Trap *p, int restore)
 	/* restore original value for exec'd kids */
 	p->flags &= ~(TF_EXEC_IGN|TF_EXEC_DFL);
 	switch (restore & SS_RESTORE_MASK) {
-	case SS_RESTORE_CURR:
-		/* leave things as they currently are */
+	case SS_RESTORE_CURR: /* leave things as they currently are */
 		break;
 	case SS_RESTORE_ORIG:
 		p->flags |= p->flags & TF_ORIG_IGN ? TF_EXEC_IGN : TF_EXEC_DFL;
