@@ -1,10 +1,10 @@
-/*	$OpenBSD: main.c,v 1.46 2010/05/19 17:36:08 jasper Exp $	*/
+/*	$OpenBSD: main.c,v 1.45 2009/01/29 23:27:26 jaredy Exp $	*/
 /*	$OpenBSD: tty.c,v 1.9 2006/03/14 22:08:01 deraadt Exp $	*/
 /*	$OpenBSD: io.c,v 1.22 2006/03/17 16:30:13 millert Exp $	*/
 /*	$OpenBSD: table.c,v 1.13 2009/01/17 22:06:44 millert Exp $	*/
 
 /*-
- * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+ * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009
  *	Thorsten Glaser <tg@mirbsd.org>
  *
  * Provided that these terms and disclaimer and all copyright notices
@@ -33,25 +33,19 @@
 #include <locale.h>
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/main.c,v 1.190 2011/04/17 12:24:43 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/main.c,v 1.158 2010/01/01 17:44:08 tg Exp $");
 
 extern char **environ;
 
-#ifndef MKSHRC_PATH
-#define MKSHRC_PATH	"~/.mkshrc"
+#if !HAVE_SETRESUGID
+extern uid_t kshuid;
+extern gid_t kshgid, kshegid;
 #endif
 
-#ifndef MKSH_DEFAULT_TMPDIR
-#define MKSH_DEFAULT_TMPDIR	"/tmp"
-#endif
-
-void chvt_reinit(void);
 static void reclaim(void);
 static void remove_temps(struct temp *);
-static mksh_uari_t rndsetup(void);
-#ifdef SIGWINCH
-static void x_sigwinch(int);
-#endif
+void chvt_reinit(void);
+Source *mksh_init(int, const char *[]);
 
 static const char initifs[] = "IFS= \t\n";
 
@@ -61,18 +55,14 @@ static const char initsubs[] =
 static const char *initcoms[] = {
 	T_typeset, "-r", initvsn, NULL,
 	T_typeset, "-x", "HOME", "PATH", "RANDOM", "SHELL", NULL,
-	T_typeset, "-i10", "COLUMNS", "KSHEGID", "KSHGID", "KSHUID", "LINES",
-	    "OPTIND", "PGRP", "PPID", "RANDOM", "SECONDS", "TMOUT", "USER_ID",
-	    NULL,
-	T_alias,
+	T_typeset, "-i10", "COLUMNS", "LINES", "OPTIND", "PGRP", "PPID",
+	    "RANDOM", "SECONDS", "TMOUT", "USER_ID", NULL,
+	"alias",
 	"integer=typeset -i",
 	T_local_typeset,
-	/* not "alias -t --": hash -r needs to work */
-	"hash=alias -t",
+	"hash=alias -t",	/* not "alias -t --": hash -r needs to work */
 	"type=whence -v",
-#if !defined(ANDROID) && !defined(MKSH_UNEMPLOYED)
-	/* not in Android for political reasons */
-	/* not in ARGE mksh due to no job control */
+#ifndef MKSH_UNEMPLOYED
 	"stop=kill -STOP",
 	"suspend=kill -STOP $$",
 #endif
@@ -81,12 +71,12 @@ static const char *initcoms[] = {
 	"history=fc -l",
 	"nameref=typeset -n",
 	"nohup=nohup ",
-	T_r_fc_e_,
+	r_fc_e_,
 	"source=PATH=$PATH:. command .",
 	"login=exec login",
 	NULL,
 	 /* this is what AT&T ksh seems to track, with the addition of emacs */
-	T_alias, "-tU",
+	"alias", "-tU",
 	"cat", "cc", "chmod", "cp", "date", "ed", "emacs", "grep", "ls",
 	"make", "mv", "pr", "rm", "sed", "sh", "vi", "who", NULL,
 	NULL
@@ -94,52 +84,9 @@ static const char *initcoms[] = {
 
 static int initio_done;
 
-/* top-level parsing and execution environment */
 static struct env env;
 struct env *e = &env;
 
-static mksh_uari_t
-rndsetup(void)
-{
-	register uint32_t h;
-	struct {
-		ALLOC_ITEM __alloc_i;
-		void *dataptr, *stkptr, *mallocptr;
-		sigjmp_buf jbuf;
-		struct timeval tv;
-		struct timezone tz;
-	} *bufptr;
-	char *cp;
-
-	cp = alloc(sizeof(*bufptr) - ALLOC_SIZE, APERM);
-#ifdef DEBUG
-	/* clear the allocated space, for valgrind */
-	memset(cp, 0, sizeof(*bufptr) - ALLOC_SIZE);
-#endif
-	/* undo what alloc() did to the malloc result address */
-	bufptr = (void *)(cp - ALLOC_SIZE);
-	/* PIE or something similar provides us with deltas here */
-	bufptr->dataptr = &rndsetupstate;
-	/* ASLR in at least Windows, Linux, some BSDs */
-	bufptr->stkptr = &bufptr;
-	/* randomised malloc in BSD (and possibly others) */
-	bufptr->mallocptr = bufptr;
-	/* glibc pointer guard */
-	sigsetjmp(bufptr->jbuf, 1);
-	/* introduce variation */
-	gettimeofday(&bufptr->tv, &bufptr->tz);
-
-	oaat1_init_impl(h);
-	/* variation through pid, ppid, and the works */
-	oaat1_addmem_impl(h, &rndsetupstate, sizeof(rndsetupstate));
-	/* some variation, some possibly entropy, depending on OE */
-	oaat1_addmem_impl(h, bufptr, sizeof(*bufptr));
-	oaat1_fini_impl(h);
-
-	afree(cp, APERM);
-	return ((mksh_uari_t)h);
-}
-
 void
 chvt_reinit(void)
 {
@@ -147,25 +94,25 @@ chvt_reinit(void)
 	ksheuid = geteuid();
 	kshpgrp = getpgrp();
 	kshppid = getppid();
+
+#if !HAVE_ARC4RANDOM
+	change_random(&kshstate_, sizeof(kshstate_));
+#endif
 }
 
-static const char *empty_argv[] = {
-	"mksh", NULL
-};
-
-int
-main(int argc, const char *argv[])
+Source *
+mksh_init(int argc, const char *argv[])
 {
 	int argi, i;
-	Source *s = NULL;
+	Source *s;
 	struct block *l;
 	unsigned char restricted, errexit, utf_flag;
-	char *cp;
-	const char *ccp, **wp;
+	const char **wp;
 	struct tbl *vp;
 	struct stat s_stdin;
 #if !defined(_PATH_DEFPATH) && defined(_CS_PATH)
-	ssize_t k;
+	size_t k;
+	char *cp;
 #endif
 
 	/* do things like getpgrp() et al. */
@@ -173,58 +120,28 @@ main(int argc, const char *argv[])
 
 	/* make sure argv[] is sane */
 	if (!*argv) {
+		static const char *empty_argv[] = {
+			"mksh", NULL
+		};
+
 		argv = empty_argv;
 		argc = 1;
 	}
-	kshname = argv[0];
+	kshname = *argv;
 
-	/* initialise permanent Area */
-	ainit(&aperm);
+	ainit(&aperm);		/* initialise permanent Area */
 
 	/* set up base environment */
 	env.type = E_NONE;
 	ainit(&env.area);
-	/* set up global l->vars and l->funs */
-	newblock();
+	newblock();		/* set up global l->vars and l->funs */
 
 	/* Do this first so output routines (eg, errorf, shellf) can work */
 	initio();
 
-	/* determine the basename (without '-' or path) of the executable */
-	ccp = kshname;
-	goto begin_parse_kshname;
-	while ((i = ccp[argi++])) {
-		if (i == '/') {
-			ccp += argi;
- begin_parse_kshname:
-			argi = 0;
-			if (*ccp == '-')
-				++ccp;
-		}
-	}
-	if (!*ccp)
-		ccp = empty_argv[0];
-
-	/* define built-in commands and see if we were called as one */
-	ktinit(&builtins, APERM,
-	    /* must be 80% of 2^n (currently 44 builtins) */ 64);
-	for (i = 0; mkshbuiltins[i].name != NULL; i++)
-		if (!strcmp(ccp, builtin(mkshbuiltins[i].name,
-		    mkshbuiltins[i].func)))
-			Flag(FAS_BUILTIN) = 1;
-
-	if (!Flag(FAS_BUILTIN)) {
-		/* check for -T option early */
-		argi = parse_args(argv, OF_FIRSTTIME, NULL);
-		if (argi < 0)
-			return (1);
-
-#ifdef MKSH_BINSHREDUCED
-		/* set FSH if we're called as -sh or /bin/sh or so */
-		if (!strcmp(ccp, "sh"))
-			change_flag(FSH, OF_FIRSTTIME, 1);
-#endif
-	}
+	argi = parse_args(argv, OF_FIRSTTIME, NULL);
+	if (argi < 0)
+		return (NULL);
 
 	initvar();
 
@@ -244,13 +161,19 @@ main(int argc, const char *argv[])
 	/* define shell keywords */
 	initkeywords();
 
+	/* define built-in commands */
+	ktinit(&builtins, APERM,
+	    /* must be 80% of 2^n (currently 44 builtins) */ 64);
+	for (i = 0; mkshbuiltins[i].name != NULL; i++)
+		builtin(mkshbuiltins[i].name, mkshbuiltins[i].func);
+
 	init_histvec();
 
 #ifdef _PATH_DEFPATH
 	def_path = _PATH_DEFPATH;
 #else
 #ifdef _CS_PATH
-	if ((k = confstr(_CS_PATH, NULL, 0)) > 0 &&
+	if ((k = confstr(_CS_PATH, NULL, 0)) != (size_t)-1 && k > 0 &&
 	    confstr(_CS_PATH, cp = alloc(k + 1, APERM), k + 1) == k + 1)
 		def_path = cp;
 	else
@@ -266,61 +189,82 @@ main(int argc, const char *argv[])
 		def_path = "/bin:/usr/bin:/sbin:/usr/sbin";
 #endif
 
-	/*
-	 * Set PATH to def_path (will set the path global variable).
+	/* Set PATH to def_path (will set the path global variable).
 	 * (import of environment below will probably change this setting).
 	 */
 	vp = global("PATH");
 	/* setstr can't fail here */
 	setstr(vp, def_path, KSH_RETURN_ERROR);
 
-	/*
-	 * Turn on nohup by default for now - will change to off
+	/* Turn on nohup by default for now - will change to off
 	 * by default once people are aware of its existence
 	 * (AT&T ksh does not have a nohup option - it always sends
 	 * the hup).
 	 */
 	Flag(FNOHUP) = 1;
 
-	/*
-	 * Turn on brace expansion by default. AT&T kshs that have
+	/* Turn on brace expansion by default. AT&T kshs that have
 	 * alternation always have it on.
 	 */
 	Flag(FBRACEEXPAND) = 1;
 
-	/*
-	 * Set edit mode to emacs by default, may be overridden
+	/* Set edit mode to emacs by default, may be overridden
 	 * by the environment or the user. Also, we want tab completion
-	 * on in vi by default.
-	 */
+	 * on in vi by default. */
 	change_flag(FEMACS, OF_SPECIAL, 1);
 #if !MKSH_S_NOVI
 	Flag(FVITABCOMPLETE) = 1;
 #endif
 
+#ifdef MKSH_BINSHREDUCED
+	/* set FSH if we're called as -sh or /bin/sh or so */
+	{
+		const char *cc;
+
+		cc = kshname;
+		i = 0; argi = 0;
+		while (cc[i] != '\0')
+			/* the following line matches '-' and '/' ;-) */
+			if ((cc[i++] | 2) == '/')
+				argi = i;
+		if (((cc[argi] | 0x20) == 's') && ((cc[argi + 1] | 0x20) == 'h'))
+			change_flag(FSH, OF_FIRSTTIME, 1);
+	}
+#endif
+
 	/* import environment */
 	if (environ != NULL)
 		for (wp = (const char **)environ; *wp != NULL; wp++)
 			typeset(*wp, IMPORT | EXPORT, 0, 0, 0);
 
-	/* for security */
-	typeset(initifs, 0, 0, 0, 0);
+	typeset(initifs, 0, 0, 0, 0);	/* for security */
 
 	/* assign default shell variable values */
 	substitute(initsubs, 0);
 
 	/* Figure out the current working directory and set $PWD */
-	vp = global("PWD");
-	cp = str_val(vp);
-	/* Try to use existing $PWD if it is valid */
-	set_current_wd((cp[0] == '/' && test_eval(NULL, TO_FILEQ, cp, ".",
-	    true)) ? cp : NULL);
-	if (current_wd[0])
-		simplify_path(current_wd);
-	/* Only set pwd if we know where we are or if it had a bogus value */
-	if (current_wd[0] || *cp)
-		/* setstr can't fail here */
-		setstr(vp, current_wd, KSH_RETURN_ERROR);
+	{
+		struct stat s_pwd, s_dot;
+		struct tbl *pwd_v = global("PWD");
+		char *pwd = str_val(pwd_v);
+		char *pwdx = pwd;
+
+		/* Try to use existing $PWD if it is valid */
+		if (pwd[0] != '/' ||
+		    stat(pwd, &s_pwd) < 0 || stat(".", &s_dot) < 0 ||
+		    s_pwd.st_dev != s_dot.st_dev ||
+		    s_pwd.st_ino != s_dot.st_ino)
+			pwdx = NULL;
+		set_current_wd(pwdx);
+		if (current_wd[0])
+			simplify_path(current_wd);
+		/* Only set pwd if we know where we are or if it had a
+		 * bogus value
+		 */
+		if (current_wd[0] || pwd != null)
+			/* setstr can't fail here */
+			setstr(pwd_v, current_wd, KSH_RETURN_ERROR);
+	}
 
 	for (wp = initcoms; *wp != NULL; wp++) {
 		shcomexec(wp);
@@ -331,10 +275,6 @@ main(int argc, const char *argv[])
 	setint(global("LINES"), 0);
 	setint(global("OPTIND"), 1);
 
-	kshuid = getuid();
-	kshgid = getgid();
-	kshegid = getegid();
-
 	safe_prompt = ksheuid ? "$ " : "# ";
 	vp = global("PS1");
 	/* Set PS1 if unset or we are root and prompt doesn't contain a # */
@@ -346,20 +286,21 @@ main(int argc, const char *argv[])
 	vp->flag |= INT_U;
 	setint((vp = global("PPID")), (mksh_uari_t)kshppid);
 	vp->flag |= INT_U;
+	setint((vp = global("RANDOM")), (mksh_uari_t)hash(kshname));
+	vp->flag |= INT_U;
 	setint((vp = global("USER_ID")), (mksh_uari_t)ksheuid);
 	vp->flag |= INT_U;
-	setint((vp = global("KSHUID")), (mksh_uari_t)kshuid);
-	vp->flag |= INT_U;
-	setint((vp = global("KSHEGID")), (mksh_uari_t)kshegid);
-	vp->flag |= INT_U;
-	setint((vp = global("KSHGID")), (mksh_uari_t)kshgid);
-	vp->flag |= INT_U;
-	setint((vp = global("RANDOM")), rndsetup());
-	vp->flag |= INT_U;
-	setint((vp_pipest = global("PIPESTATUS")), 0);
+#if HAVE_ARC4RANDOM
+	Flag(FARC4RANDOM) = 1;		/* initialised */
+#endif
 
 	/* Set this before parsing arguments */
-	Flag(FPRIVILEGED) = kshuid != ksheuid || kshgid != kshegid;
+#if HAVE_SETRESUGID
+	Flag(FPRIVILEGED) = getuid() != ksheuid || getgid() != getegid();
+#else
+	Flag(FPRIVILEGED) = (kshuid = getuid()) != ksheuid ||
+	    (kshgid = getgid()) != (kshegid = getegid());
+#endif
 
 	/* this to note if monitor is set on command line (see below) */
 #ifndef MKSH_UNEMPLOYED
@@ -368,25 +309,20 @@ main(int argc, const char *argv[])
 	/* this to note if utf-8 mode is set on command line (see below) */
 	UTFMODE = 2;
 
-	if (!Flag(FAS_BUILTIN)) {
-		argi = parse_args(argv, OF_CMDLINE, NULL);
-		if (argi < 0)
-			return (1);
-	}
+	argi = parse_args(argv, OF_CMDLINE, NULL);
+	if (argi < 0)
+		return (NULL);
 
 	/* process this later only, default to off (hysterical raisins) */
 	utf_flag = UTFMODE;
 	UTFMODE = 0;
 
-	if (Flag(FAS_BUILTIN)) {
-		/* auto-detect from environment variables, always */
-		utf_flag = 3;
-	} else if (Flag(FCOMMAND)) {
+	if (Flag(FCOMMAND)) {
 		s = pushs(SSTRING, ATEMP);
 		if (!(s->start = s->str = argv[argi++]))
-			errorf("%s %s", "-c", "requires an argument");
+			errorf("-c requires an argument");
 #ifdef MKSH_MIDNIGHTBSD01ASH_COMPAT
-		/* compatibility to MidnightBSD 0.1 /bin/sh (kludge) */
+		/* compatibility to MidnightBSD 0.1 /bin/sh (not desired) */
 		if (Flag(FSH) && argv[argi] && !strcmp(argv[argi], "--"))
 			++argi;
 #endif
@@ -398,7 +334,7 @@ main(int argc, const char *argv[])
 		s->u.shf = shf_open(s->file, O_RDONLY, 0,
 		    SHF_MAPHI | SHF_CLEXEC);
 		if (s->u.shf == NULL) {
-			shl_stdout_ok = false;
+			shl_stdout_ok = 0;
 			warningf(true, "%s: %s", s->file, strerror(errno));
 			/* mandated by SUSv4 */
 			exstat = 127;
@@ -427,85 +363,47 @@ main(int argc, const char *argv[])
 
 	/* initialise job control */
 	j_init();
+	/* set: 0/1; unset: 2->0 */
+	UTFMODE = utf_flag & 1;
 	/* Do this after j_init(), as tty_fd is not initialised until then */
 	if (Flag(FTALKING)) {
 		if (utf_flag == 2) {
 #ifndef MKSH_ASSUME_UTF8
-			/* auto-detect from locale or environment */
-			utf_flag = 4;
-#elif MKSH_ASSUME_UTF8
-			utf_flag = 1;
+#define isuc(x)	(((x) != NULL) && \
+		    (stristr((x), "UTF-8") || stristr((x), "utf8")))
+		/* Check if we're in a UTF-8 locale */
+			const char *ccp;
+
+#if HAVE_SETLOCALE_CTYPE
+			ccp = setlocale(LC_CTYPE, "");
+#if HAVE_LANGINFO_CODESET
+			if (!isuc(ccp))
+				ccp = nl_langinfo(CODESET);
+#endif
 #else
-			/* always disable UTF-8 (for interactive) */
-			utf_flag = 0;
+			/* these were imported from environ earlier */
+			ccp = str_val(global("LC_ALL"));
+			if (ccp == null)
+				ccp = str_val(global("LC_CTYPE"));
+			if (ccp == null)
+				ccp = str_val(global("LANG"));
+#endif
+			UTFMODE = isuc(ccp);
+#undef isuc
+#elif MKSH_ASSUME_UTF8
+			UTFMODE = 1;
+#else
+			UTFMODE = 0;
 #endif
 		}
 		x_init();
 	}
 
-#ifdef SIGWINCH
-	sigtraps[SIGWINCH].flags |= TF_SHELL_USES;
-	setsig(&sigtraps[SIGWINCH], x_sigwinch,
-	    SS_RESTORE_ORIG|SS_FORCE|SS_SHTRAP);
-#endif
-
 	l = e->loc;
-	if (Flag(FAS_BUILTIN)) {
-		l->argc = argc;
-		l->argv = argv;
-		l->argv[0] = ccp;
-	} else {
-		l->argc = argc - argi;
-		l->argv = &argv[argi - 1];
-		l->argv[0] = kshname;
-		getopts_reset(1);
-	}
-
-	/* divine the initial state of the utf8-mode Flag */
-#define isuc(x)	(((x) != NULL) && \
-	    (stristr((x), "UTF-8") || stristr((x), "utf8")))
-	ccp = null;
-	switch (utf_flag) {
-
-	/* auto-detect from locale or environment */
-	case 4:
-#if HAVE_SETLOCALE_CTYPE
-		ccp = setlocale(LC_CTYPE, "");
-#if HAVE_LANGINFO_CODESET
-		if (!isuc(ccp))
-			ccp = nl_langinfo(CODESET);
-#endif
-		if (!isuc(ccp))
-			ccp = null;
-		/* FALLTHROUGH */
-#endif
-
-	/* auto-detect from environment */
-	case 3:
-		/* these were imported from environ earlier */
-		if (ccp == null)
-			ccp = str_val(global("LC_ALL"));
-		if (ccp == null)
-			ccp = str_val(global("LC_CTYPE"));
-		if (ccp == null)
-			ccp = str_val(global("LANG"));
-		UTFMODE = isuc(ccp);
-		break;
-
-	/* not set on command line, not FTALKING */
-	case 2:
-	/* unknown values */
-	default:
-		utf_flag = 0;
-		/* FALLTHROUGH */
-
-	/* known values */
-	case 1:
-	case 0:
-		UTFMODE = utf_flag;
-		break;
-	}
-#undef isuc
+	l->argv = &argv[argi - 1];
+	l->argc = argc - argi;
+	l->argv[0] = kshname;
+	getopts_reset(1);
 
 	/* Disable during .profile/ENV reading */
 	restricted = Flag(FRESTRICTED);
@@ -513,12 +411,11 @@ main(int argc, const char *argv[])
 	errexit = Flag(FERREXIT);
 	Flag(FERREXIT) = 0;
 
-	/*
-	 * Do this before profile/$ENV so that if it causes problems in them,
+	/* Do this before profile/$ENV so that if it causes problems in them,
 	 * user will know why things broke.
 	 */
 	if (!current_wd[0] && Flag(FTALKING))
-		warningf(false, "can't determine current directory");
+		warningf(false, "Cannot determine current working directory");
 
 	if (Flag(FLOGIN)) {
 		include(KSH_SYSTEM_PROFILE, 0, NULL, 1);
@@ -532,7 +429,7 @@ main(int argc, const char *argv[])
 		char *env_file;
 
 		/* include $ENV */
-		env_file = substitute(substitute("${ENV:-" MKSHRC_PATH "}", 0),
+		env_file = substitute(substitute("${ENV:-~/.mkshrc}", 0),
 		    DOTILDE);
 		if (*env_file != '\0')
 			include(env_file, 0, NULL, 1);
@@ -550,20 +447,24 @@ main(int argc, const char *argv[])
 	}
 	Flag(FERREXIT) = errexit;
 
-	if (Flag(FTALKING))
+	if (Flag(FTALKING)) {
 		hist_init(s);
-	else
-		/* set after ENV */
-		Flag(FTRACKALL) = 1;
+		alarm_init();
+	} else
+		Flag(FTRACKALL) = 1;	/* set after ENV */
 
-	alarm_init();
+	return (s);
+}
 
-	if (Flag(FAS_BUILTIN))
-		return (shcomexec(l->argv));
+int
+main(int argc, const char *argv[])
+{
+	Source *s;
 
-	/* doesn't return */
-	shell(s, true);
-	return (0);
+	if ((s = mksh_init(argc, argv)))
+		/* doesn’t return */
+		shell(s, true);
+	return (1);
 }
 
 int
@@ -597,23 +498,21 @@ include(const char *name, int argc, const char **argv, int intr_ok)
 		switch (i) {
 		case LRETURN:
 		case LERROR:
-			/* see below */
-			return (exstat & 0xFF);
+			return (exstat & 0xff); /* see below */
 		case LINTR:
-			/*
-			 * intr_ok is set if we are including .profile or $ENV.
+			/* intr_ok is set if we are including .profile or $ENV.
 			 * If user ^Cs out, we don't want to kill the shell...
 			 */
 			if (intr_ok && (exstat - 128) != SIGTERM)
 				return (1);
-			/* FALLTHROUGH */
+			/* FALLTHRU */
 		case LEXIT:
 		case LLEAVE:
 		case LSHELL:
 			unwind(i);
 			/* NOTREACHED */
 		default:
-			internal_errorf("%s %d", "include", i);
+			internal_errorf("include: %d", i);
 			/* NOTREACHED */
 		}
 	}
@@ -630,19 +529,16 @@ include(const char *name, int argc, const char **argv, int intr_ok)
 		e->loc->argv = old_argv;
 		e->loc->argc = old_argc;
 	}
-	/* & 0xff to ensure value not -1 */
-	return (i & 0xFF);
+	return (i & 0xff);	/* & 0xff to ensure value not -1 */
 }
 
-/* spawn a command into a shell optionally keeping track of the line number */
 int
-command(const char *comm, int line)
+command(const char *comm)
 {
 	Source *s;
 
 	s = pushs(SSTRING, ATEMP);
 	s->start = s->str = comm;
-	s->line = line;
 	return (shell(s, false));
 }
 
@@ -656,32 +552,30 @@ shell(Source * volatile s, volatile int toplevel)
 	volatile int wastty = s->flags & SF_TTY;
 	volatile int attempts = 13;
 	volatile int interactive = Flag(FTALKING) && toplevel;
-	volatile bool sfirst = true;
 	Source *volatile old_source = source;
 	int i;
 
+	s->flags |= SF_FIRST;	/* enable UTF-8 BOM check */
+
 	newenv(E_PARSE);
 	if (interactive)
 		really_exit = 0;
 	i = sigsetjmp(e->jbuf, 0);
 	if (i) {
 		switch (i) {
-		case LINTR:
-			/* we get here if SIGINT not caught or ignored */
+		case LINTR: /* we get here if SIGINT not caught or ignored */
 		case LERROR:
 		case LSHELL:
 			if (interactive) {
 				if (i == LINTR)
 					shellf("\n");
-				/*
-				 * Reset any eof that was read as part of a
+				/* Reset any eof that was read as part of a
 				 * multiline command.
 				 */
 				if (Flag(FIGNOREEOF) && s->type == SEOF &&
 				    wastty)
 					s->type = SSTDIN;
-				/*
-				 * Used by exit command to get back to
+				/* Used by exit command to get back to
 				 * top level shell. Kind of strange since
 				 * interactive is set if we are reading from
 				 * a tty, but to have stopped jobs, one only
@@ -691,23 +585,22 @@ shell(Source * volatile s, volatile int toplevel)
 				s->start = s->str = null;
 				break;
 			}
-			/* FALLTHROUGH */
+			/* FALLTHRU */
 		case LEXIT:
 		case LLEAVE:
 		case LRETURN:
 			source = old_source;
 			quitenv(NULL);
-			/* keep on going */
-			unwind(i);
+			unwind(i);	/* keep on going */
 			/* NOTREACHED */
 		default:
 			source = old_source;
 			quitenv(NULL);
-			internal_errorf("%s %d", "shell", i);
+			internal_errorf("shell: %d", i);
 			/* NOTREACHED */
 		}
 	}
-	while (/* CONSTCOND */ 1) {
+	while (1) {
 		if (trap)
 			runtraps(0);
 
@@ -721,19 +614,17 @@ shell(Source * volatile s, volatile int toplevel)
 			j_notify();
 			set_prompt(PS1, s);
 		}
-		t = compile(s, sfirst);
-		sfirst = false;
+		t = compile(s);
 		if (t != NULL && t->type == TEOF) {
 			if (wastty && Flag(FIGNOREEOF) && --attempts > 0) {
-				shellf("Use 'exit' to leave mksh\n");
+				shellf("Use 'exit' to leave ksh\n");
 				s->type = SSTDIN;
 			} else if (wastty && !really_exit &&
 			    j_stopped_running()) {
 				really_exit = 1;
 				s->type = SSTDIN;
 			} else {
-				/*
-				 * this for POSIX which says EXIT traps
+				/* this for POSIX which says EXIT traps
 				 * shall be taken in the environment
 				 * immediately after the last command
 				 * executed.
@@ -763,17 +654,13 @@ unwind(int i)
 	/* ordering for EXIT vs ERR is a bit odd (this is what AT&T ksh does) */
 	if (i == LEXIT || (Flag(FERREXIT) && (i == LERROR || i == LINTR) &&
 	    sigtraps[SIGEXIT_].trap)) {
-		++trap_nested;
-		runtrap(&sigtraps[SIGEXIT_], trap_nested == 1);
-		--trap_nested;
+		runtrap(&sigtraps[SIGEXIT_]);
 		i = LLEAVE;
 	} else if (Flag(FERREXIT) && (i == LERROR || i == LINTR)) {
-		++trap_nested;
-		runtrap(&sigtraps[SIGERR_], trap_nested == 1);
-		--trap_nested;
+		runtrap(&sigtraps[SIGERR_]);
 		i = LLEAVE;
 	}
-	while (/* CONSTCOND */ 1) {
+	while (1) {
 		switch (e->type) {
 		case E_PARSE:
 		case E_FUNC:
@@ -785,7 +672,7 @@ unwind(int i)
 		case E_NONE:
 			if (i == LINTR)
 				e->flags |= EF_FAKE_SIGDIE;
-			/* FALLTHROUGH */
+			/* FALLTHRU */
 		default:
 			quitenv(NULL);
 		}
@@ -803,8 +690,7 @@ newenv(int type)
 	 * so first get the actually used memory, then assign it
 	 */
 	cp = alloc(sizeof(struct env) - ALLOC_SIZE, ATEMP);
-	/* undo what alloc() did to the malloc result address */
-	ep = (void *)(cp - ALLOC_SIZE);
+	ep = (void *)(cp - ALLOC_SIZE);	/* undo what alloc() did */
 	/* initialise public members of struct env (not the ALLOC_ITEM) */
 	ainit(&ep->area);
 	ep->oenv = e;
@@ -831,17 +717,14 @@ quitenv(struct shf *shf)
 			/* if ep->savefd[fd] < 0, means fd was closed */
 			if (ep->savefd[fd])
 				restfd(fd, ep->savefd[fd]);
-		if (ep->savefd[2])
-			/* Clear any write errors */
+		if (ep->savefd[2])	/* Clear any write errors */
 			shf_reopen(2, SHF_WR, shl_out);
 	}
-	/*
-	 * Bottom of the stack.
+	/* Bottom of the stack.
 	 * Either main shell is exiting or cleanup_parents_env() was called.
 	 */
 	if (ep->oenv == NULL) {
-		if (ep->type == E_NONE) {
-			/* Main shell exiting? */
+		if (ep->type == E_NONE) {	/* Main shell exiting? */
 #if HAVE_PERSISTENT_HISTORY
 			if (Flag(FTALKING))
 				hist_finish();
@@ -850,8 +733,7 @@ quitenv(struct shf *shf)
 			if (ep->flags & EF_FAKE_SIGDIE) {
 				int sig = exstat - 128;
 
-				/*
-				 * ham up our death a bit (AT&T ksh
+				/* ham up our death a bit (AT&T ksh
 				 * only seems to do this for SIGTERM)
 				 * Don't do it for SIGQUIT, since we'd
 				 * dump a core..
@@ -935,8 +817,7 @@ remove_temps(struct temp *tp)
 			unlink(tp->name);
 }
 
-/*
- * Initialise tty_fd. Used for saving/reseting tty modes upon
+/* Initialise tty_fd. Used for saving/reseting tty modes upon
  * foreground job completion and for setting up tty process group.
  */
 void
@@ -949,21 +830,19 @@ tty_init(bool init_ttystate, bool need_tty)
 		close(tty_fd);
 		tty_fd = -1;
 	}
-	tty_devtty = true;
+	tty_devtty = 1;
 
 #ifdef _UWIN
-	/*XXX imake style */
-	if (isatty(3)) {
-		/* fd 3 on UWIN _is_ /dev/tty (or our controlling tty) */
+	/* XXX imake style */
+	if (isatty(3))
 		tfd = 3;
-		do_close = false;
-	} else
+	else
 #endif
-	  if ((tfd = open("/dev/tty", O_RDWR, 0)) < 0) {
-		tty_devtty = false;
+	if ((tfd = open("/dev/tty", O_RDWR, 0)) < 0) {
+		tty_devtty = 0;
 		if (need_tty)
-			warningf(false, "%s: %s %s: %s",
-			    "No controlling tty", "open", "/dev/tty",
+			warningf(false,
+			    "No controlling tty (open /dev/tty: %s)",
 			    strerror(errno));
 	}
 	if (tfd < 0) {
@@ -974,18 +853,20 @@ tty_init(bool init_ttystate, bool need_tty)
 			tfd = 2;
 		else {
 			if (need_tty)
-				warningf(false, "can't find tty fd");
+				warningf(false,
+				    "Can't find tty file descriptor");
 			return;
 		}
 	}
 	if ((tty_fd = fcntl(tfd, F_DUPFD, FDBASE)) < 0) {
 		if (need_tty)
-			warningf(false, "%s: %s %s: %s", "j_ttyinit",
-			    "dup of tty fd", "failed", strerror(errno));
+			warningf(false, "j_ttyinit: dup of tty fd failed: %s",
+			    strerror(errno));
 	} else if (fcntl(tty_fd, F_SETFD, FD_CLOEXEC) < 0) {
 		if (need_tty)
-			warningf(false, "%s: %s: %s", "j_ttyinit",
-			    "can't set close-on-exec flag", strerror(errno));
+			warningf(false,
+			    "j_ttyinit: can't set close-on-exec flag: %s",
+			    strerror(errno));
 		close(tty_fd);
 		tty_fd = -1;
 	} else if (init_ttystate)
@@ -1004,59 +885,21 @@ tty_close(void)
 }
 
 /* A shell error occurred (eg, syntax error, etc.) */
-
-#define VWARNINGF_ERRORPREFIX	1
-#define VWARNINGF_FILELINE	2
-#define VWARNINGF_BUILTIN	4
-#define VWARNINGF_INTERNAL	8
-
-static void MKSH_A_FORMAT(__printf__, 2, 0)
-vwarningf(unsigned int flags, const char *fmt, va_list ap)
-{
-	if (*fmt != 1) {
-		if (flags & VWARNINGF_INTERNAL)
-			shf_fprintf(shl_out, "internal error: ");
-		if (flags & VWARNINGF_ERRORPREFIX)
-			error_prefix(tobool(flags & VWARNINGF_FILELINE));
-		if ((flags & VWARNINGF_BUILTIN) &&
-		    /* not set when main() calls parse_args() */
-		    builtin_argv0 && builtin_argv0 != kshname)
-			shf_fprintf(shl_out, "%s: ", builtin_argv0);
-		shf_vfprintf(shl_out, fmt, ap);
-		shf_putchar('\n', shl_out);
-	}
-	shf_flush(shl_out);
-}
-
-void
-errorfx(int rc, const char *fmt, ...)
-{
-	va_list va;
-
-	exstat = rc;
-
-	/* debugging: note that stdout not valid */
-	shl_stdout_ok = false;
-
-	va_start(va, fmt);
-	vwarningf(VWARNINGF_ERRORPREFIX | VWARNINGF_FILELINE, fmt, va);
-	va_end(va);
-	unwind(LERROR);
-}
-
 void
 errorf(const char *fmt, ...)
 {
 	va_list va;
 
+	shl_stdout_ok = 0;	/* debugging: note that stdout not valid */
 	exstat = 1;
-
-	/* debugging: note that stdout not valid */
-	shl_stdout_ok = false;
-
-	va_start(va, fmt);
-	vwarningf(VWARNINGF_ERRORPREFIX | VWARNINGF_FILELINE, fmt, va);
-	va_end(va);
+	if (*fmt != 1) {
+		error_prefix(true);
+		va_start(va, fmt);
+		shf_vfprintf(shl_out, fmt, va);
+		va_end(va);
+		shf_putchar('\n', shl_out);
+	}
+	shf_flush(shl_out);
 	unwind(LERROR);
 }
 
@@ -1066,14 +909,15 @@ warningf(bool fileline, const char *fmt, ...)
 {
 	va_list va;
 
+	error_prefix(fileline);
 	va_start(va, fmt);
-	vwarningf(VWARNINGF_ERRORPREFIX | (fileline ? VWARNINGF_FILELINE : 0),
-	    fmt, va);
+	shf_vfprintf(shl_out, fmt, va);
 	va_end(va);
+	shf_putchar('\n', shl_out);
+	shf_flush(shl_out);
 }
 
-/*
- * Used by built-in utilities to prefix shell and utility name to message
+/* Used by built-in utilities to prefix shell and utility name to message
  * (also unwinds environments for special builtins).
  */
 void
@@ -1081,18 +925,20 @@ bi_errorf(const char *fmt, ...)
 {
 	va_list va;
 
-	/* debugging: note that stdout not valid */
-	shl_stdout_ok = false;
-
+	shl_stdout_ok = 0;	/* debugging: note that stdout not valid */
 	exstat = 1;
-
-	va_start(va, fmt);
-	vwarningf(VWARNINGF_ERRORPREFIX | VWARNINGF_FILELINE |
-	    VWARNINGF_BUILTIN, fmt, va);
-	va_end(va);
-
-	/*
-	 * POSIX special builtins and ksh special builtins cause
+	if (*fmt != 1) {
+		error_prefix(true);
+		/* not set when main() calls parse_args() */
+		if (builtin_argv0)
+			shf_fprintf(shl_out, "%s: ", builtin_argv0);
+		va_start(va, fmt);
+		shf_vfprintf(shl_out, fmt, va);
+		va_end(va);
+		shf_putchar('\n', shl_out);
+	}
+	shf_flush(shl_out);
+	/* POSIX special builtins and ksh special builtins cause
 	 * non-interactive shells to exit.
 	 * XXX odd use of KEEPASN; also may not want LERROR here
 	 */
@@ -1103,13 +949,22 @@ bi_errorf(const char *fmt, ...)
 }
 
 /* Called when something that shouldn't happen does */
+void
+internal_verrorf(const char *fmt, va_list ap)
+{
+	shf_fprintf(shl_out, "internal error: ");
+	shf_vfprintf(shl_out, fmt, ap);
+	shf_putchar('\n', shl_out);
+	shf_flush(shl_out);
+}
+
 void
 internal_errorf(const char *fmt, ...)
 {
 	va_list va;
 
 	va_start(va, fmt);
-	vwarningf(VWARNINGF_INTERNAL, fmt, va);
+	internal_verrorf(fmt, va);
 	va_end(va);
 	unwind(LERROR);
 }
@@ -1120,7 +975,7 @@ internal_warningf(const char *fmt, ...)
 	va_list va;
 
 	va_start(va, fmt);
-	vwarningf(VWARNINGF_INTERNAL, fmt, va);
+	internal_verrorf(fmt, va);
 	va_end(va);
 }
 
@@ -1145,8 +1000,7 @@ shellf(const char *fmt, ...)
 {
 	va_list va;
 
-	if (!initio_done)
-		/* shl_out may not be set up yet... */
+	if (!initio_done) /* shl_out may not be set up yet... */
 		return;
 	va_start(va, fmt);
 	shf_vfprintf(shl_out, fmt, va);
@@ -1182,11 +1036,9 @@ struct shf shf_iob[3];
 void
 initio(void)
 {
-	/* force buffer allocation */
-	shf_fdopen(1, SHF_WR, shl_stdout);
+	shf_fdopen(1, SHF_WR, shl_stdout);	/* force buffer allocation */
 	shf_fdopen(2, SHF_WR, shl_out);
-	/* force buffer allocation */
-	shf_fdopen(2, SHF_WR, shl_spare);
+	shf_fdopen(2, SHF_WR, shl_spare);	/* force buffer allocation */
 	initio_done = 1;
 }
 
@@ -1200,7 +1052,7 @@ ksh_dup2(int ofd, int nfd, bool errok)
 		errorf("too many files open in shell");
 
 #ifdef __ultrix
-	/*XXX imake style */
+	/* XXX imake style */
 	if (rv >= 0)
 		fcntl(nfd, F_SETFD, 0);
 #endif
@@ -1209,8 +1061,8 @@ ksh_dup2(int ofd, int nfd, bool errok)
 }
 
 /*
- * Move fd from user space (0 <= fd < 10) to shell space (fd >= 10),
- * set close-on-exec flag. See FDBASE in sh.h, maybe 24 not 10 here.
+ * move fd from user space (0<=fd<10) to shell space (fd>=10),
+ * set close-on-exec flag.
  */
 short
 savefd(int fd)
@@ -1231,12 +1083,10 @@ restfd(int fd, int ofd)
 {
 	if (fd == 2)
 		shf_flush(&shf_iob[fd]);
-	if (ofd < 0)
-		/* original fd closed */
+	if (ofd < 0)		/* original fd closed */
 		close(fd);
 	else if (fd != ofd) {
-		/*XXX: what to do if this dup fails? */
-		ksh_dup2(ofd, fd, true);
+		ksh_dup2(ofd, fd, true); /* XXX: what to do if this fails? */
 		close(ofd);
 	}
 }
@@ -1263,8 +1113,7 @@ closepipe(int *pv)
 	close(pv[1]);
 }
 
-/*
- * Called by iosetup() (deals with 2>&4, etc.), c_read, c_print to turn
+/* Called by iosetup() (deals with 2>&4, etc.), c_read, c_print to turn
  * a string (the X in 2>&X, read -uX, print -uX) into a file descriptor.
  */
 int
@@ -1287,8 +1136,7 @@ check_fd(const char *name, int mode, const char **emsgp)
 		return (-1);
 	}
 	fl &= O_ACCMODE;
-	/*
-	 * X_OK is a kludge to disable this check for dups (x<&1):
+	/* X_OK is a kludge to disable this check for dups (x<&1):
 	 * historical shells never did this check (XXX don't know what
 	 * POSIX has to say).
 	 */
@@ -1324,8 +1172,7 @@ coproc_read_close(int fd)
 	}
 }
 
-/*
- * Called by c_read() and by iosetup() to close the other side of the
+/* Called by c_read() and by iosetup() to close the other side of the
  * read pipe, so reads will actually terminate.
  */
 void
@@ -1337,8 +1184,7 @@ coproc_readw_close(int fd)
 	}
 }
 
-/*
- * Called by c_print when a write to a fd fails with EPIPE and by iosetup
+/* Called by c_print when a write to a fd fails with EPIPE and by iosetup
  * when co-process input is dup'd
  */
 void
@@ -1350,8 +1196,7 @@ coproc_write_close(int fd)
 	}
 }
 
-/*
- * Called to check for existence of/value of the co-process file descriptor.
+/* Called to check for existence of/value of the co-process file descriptor.
  * (Used by check_fd() and by c_read/c_print to deal with -p option).
  */
 int
@@ -1366,8 +1211,7 @@ coproc_getfd(int mode, const char **emsgp)
 	return (-1);
 }
 
-/*
- * called to close file descriptors related to the coprocess (if any)
+/* called to close file descriptors related to the coprocess (if any)
  * Should be called with SIGCHLD blocked.
  */
 void
@@ -1399,14 +1243,13 @@ maketemp(Area *ap, Temp_type type, struct temp **tlist)
 	char *pathname;
 	const char *dir;
 
-	dir = tmpdir ? tmpdir : MKSH_DEFAULT_TMPDIR;
+	dir = tmpdir ? tmpdir : "/tmp";
 #if HAVE_MKSTEMP
 	len = strlen(dir) + 6 + 10 + 1;
 #else
 	pathname = tempnam(dir, "mksh.");
 	len = ((pathname == NULL) ? 0 : strlen(pathname)) + 1;
 #endif
-	/* reasonably sure that this will not overflow */
 	tp = alloc(sizeof(struct temp) + len, ap);
 	tp->name = (char *)&tp[1];
 #if !HAVE_MKSTEMP
@@ -1414,14 +1257,14 @@ maketemp(Area *ap, Temp_type type, struct temp **tlist)
 		tp->name[0] = '\0';
 	else {
 		memcpy(tp->name, pathname, len);
-		free_ostempnam(pathname);
+		free(pathname);
 	}
 #endif
 	pathname = tp->name;
 	tp->shf = NULL;
 	tp->type = type;
 #if HAVE_MKSTEMP
-	shf_snprintf(pathname, len, "%s%s", dir, "/mksh.XXXXXXXXXX");
+	shf_snprintf(pathname, len, "%s/mksh.XXXXXXXXXX", dir);
 	if ((fd = mkstemp(pathname)) >= 0)
 #else
 	if (tp->name[0] && (fd = open(tp->name, O_CREAT | O_RDWR, 0600)) >= 0)
@@ -1447,6 +1290,26 @@ static int tnamecmp(const void *, const void *);
 static struct tbl *ktscan(struct table *, const char *, uint32_t,
     struct tbl ***);
 
+/* Bob Jenkins' one-at-a-time hash */
+uint32_t
+oaathash_full(register const uint8_t *bp)
+{
+	register uint32_t h = 0;
+	register uint8_t c;
+
+	while ((c = *bp++)) {
+		h += c;
+		h += h << 10;
+		h ^= h >> 6;
+	}
+
+	h += h << 3;
+	h ^= h >> 11;
+	h += h << 15;
+
+	return (h);
+}
+
 static void
 texpand(struct table *tp, size_t nsize)
 {
@@ -1454,18 +1317,15 @@ texpand(struct table *tp, size_t nsize)
 	struct tbl *tblp, **pp;
 	struct tbl **ntblp, **otblp = tp->tbls;
 
-	ntblp = alloc2(nsize, sizeof(struct tbl *), tp->areap);
+	ntblp = alloc(nsize * sizeof(struct tbl *), tp->areap);
 	for (i = 0; i < nsize; i++)
 		ntblp[i] = NULL;
 	tp->size = nsize;
-	/* table can get 80% full */
-	tp->nfree = (nsize * 4) / 5;
+	tp->nfree = (nsize * 4) / 5;	/* table can get 80% full */
 	tp->tbls = ntblp;
 	if (otblp == NULL)
 		return;
-
-	/* from here on nsize := mask */
-	nsize--;
+	nsize--;			/* from here on nsize := mask */
 	for (i = 0; i < osize; i++)
 		if ((tblp = otblp[i]) != NULL) {
 			if ((tblp->flag & DEFINED)) {
@@ -1536,7 +1396,7 @@ struct tbl *
 ktenter(struct table *tp, const char *n, uint32_t h)
 {
 	struct tbl **pp, *p;
-	size_t len;
+	int len;
 
 	if (tp->size == 0)
 		texpand(tp, INIT_TBLS);
@@ -1551,12 +1411,14 @@ ktenter(struct table *tp, const char *n, uint32_t h)
 	}
 
 	/* create new tbl entry */
-	len = strlen(n);
-	checkoktoadd(len, offsetof(struct tbl, name[0]) + 1);
-	p = alloc(offsetof(struct tbl, name[0]) + ++len, tp->areap);
+	len = strlen(n) + 1;
+	p = alloc(offsetof(struct tbl, name[0]) + len, tp->areap);
 	p->flag = 0;
 	p->type = 0;
 	p->areap = tp->areap;
+#ifdef notyet_ktremove
+	p->tablep = tp;
+#endif
 	p->ua.hval = h;
 	p->u2.field = 0;
 	p->u.array = NULL;
@@ -1568,6 +1430,28 @@ ktenter(struct table *tp, const char *n, uint32_t h)
 	return (p);
 }
 
+#ifdef notyet_ktremove
+void
+ktremove(struct tbl *p)
+{
+	struct tbl **pp;
+
+	if (p->tablep && p->tablep->size && ktscan(p->tablep, p->name,
+	    p->ua.hval, &pp) == p) {
+		/* ktremove p */
+wontwork("cannot use NULL here, see r1.143 commit message");
+		*pp = NULL;
+		p->tablep->nfree++;
+		/* get rid of p */
+wontwork("need to check FINUSE, see texpand");
+		afree(p, p->areap);
+	} else {
+		/* mark p as free for garbage collection via texpand */
+		p->flag = 0;
+	}
+}
+#endif
+
 void
 ktwalk(struct tstate *ts, struct table *tp)
 {
@@ -1601,8 +1485,7 @@ ktsort(struct table *tp)
 	size_t i;
 	struct tbl **p, **sp, **dp;
 
-	/* tp->size + 1 will not overflow */
-	p = alloc2(tp->size + 1, sizeof(struct tbl *), ATEMP);
+	p = alloc((tp->size + 1) * sizeof(struct tbl *), ATEMP);
 	sp = tp->tbls;		/* source */
 	dp = p;			/* dest */
 	i = (size_t)tp->size;
@@ -1614,13 +1497,3 @@ ktsort(struct table *tp)
 	p[i] = NULL;
 	return (p);
 }
-
-#ifdef SIGWINCH
-static void
-x_sigwinch(int sig MKSH_A_UNUSED)
-{
-	/* this runs inside interrupt context, with errno saved */
-
-	got_winch = 1;
-}
-#endif
