@@ -1,15 +1,15 @@
-/*	$OpenBSD: sh.h,v 1.30 2010/01/04 18:07:11 deraadt Exp $	*/
+/*	$OpenBSD: sh.h,v 1.29 2005/12/11 18:53:51 deraadt Exp $	*/
 /*	$OpenBSD: shf.h,v 1.6 2005/12/11 18:53:51 deraadt Exp $	*/
 /*	$OpenBSD: table.h,v 1.7 2005/12/11 20:31:21 otto Exp $	*/
 /*	$OpenBSD: tree.h,v 1.10 2005/03/28 21:28:22 deraadt Exp $	*/
 /*	$OpenBSD: expand.h,v 1.6 2005/03/30 17:16:37 deraadt Exp $	*/
 /*	$OpenBSD: lex.h,v 1.11 2006/05/29 18:22:24 otto Exp $	*/
-/*	$OpenBSD: proto.h,v 1.33 2010/05/19 17:36:08 jasper Exp $	*/
+/*	$OpenBSD: proto.h,v 1.32 2009/01/29 23:27:26 jaredy Exp $	*/
 /*	$OpenBSD: c_test.h,v 1.4 2004/12/20 11:34:26 otto Exp $	*/
 /*	$OpenBSD: tty.h,v 1.5 2004/12/20 11:34:26 otto Exp $	*/
 
 /*-
- * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+ * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
  *	Thorsten Glaser <tg@mirbsd.org>
  *
  * Provided that these terms and disclaimer and all copyright notices
@@ -68,6 +68,9 @@
 #include <setjmp.h>
 #include <signal.h>
 #include <stdarg.h>
+#if HAVE_STDBOOL_H
+#include <stdbool.h>
+#endif
 #include <stddef.h>
 #if HAVE_STDINT_H
 #include <stdint.h>
@@ -90,12 +93,12 @@
 
 #undef __attribute__
 #if HAVE_ATTRIBUTE_BOUNDED
-#define MKSH_A_BOUNDED(x,y,z)	__attribute__((__bounded__ (x, y, z)))
+#define MKSH_A_BOUNDED(x,y,z)	__attribute__((bounded (x, y, z)))
 #else
 #define MKSH_A_BOUNDED(x,y,z)	/* nothing */
 #endif
 #if HAVE_ATTRIBUTE_FORMAT
-#define MKSH_A_FORMAT(x,y,z)	__attribute__((__format__ (x, y, z)))
+#define MKSH_A_FORMAT(x,y,z)	__attribute__((format (x, y, z)))
 #else
 #define MKSH_A_FORMAT(x,y,z)	/* nothing */
 #endif
@@ -105,24 +108,22 @@
 #define MKSH_A_NONNULL(a)	/* nothing */
 #endif
 #if HAVE_ATTRIBUTE_NORETURN
-#define MKSH_A_NORETURN		__attribute__((__noreturn__))
+#define MKSH_A_NORETURN		__attribute__((noreturn))
 #else
 #define MKSH_A_NORETURN		/* nothing */
 #endif
 #if HAVE_ATTRIBUTE_UNUSED
-#define MKSH_A_UNUSED		__attribute__((__unused__))
+#define MKSH_A_UNUSED		__attribute__((unused))
 #else
 #define MKSH_A_UNUSED		/* nothing */
 #endif
 #if HAVE_ATTRIBUTE_USED
-#define MKSH_A_USED		__attribute__((__used__))
+#define MKSH_A_USED		__attribute__((used))
 #else
 #define MKSH_A_USED		/* nothing */
 #endif
 
-#if defined(MirBSD) && (MirBSD >= 0x09A1) && \
-    defined(__ELF__) && defined(__GNUC__) && \
-    !defined(__llvm__) && !defined(__NWCC__)
+#if (defined(MirBSD) && (MirBSD >= 0x09A1))
 /*
  * We got usable __IDSTRING __COPYRIGHT __RCSID __SCCSID macros
  * which work for all cases; no need to redefine them using the
@@ -138,22 +139,18 @@
 #undef __SCCSID
 #define __IDSTRING_CONCAT(l,p)		__LINTED__ ## l ## _ ## p
 #define __IDSTRING_EXPAND(l,p)		__IDSTRING_CONCAT(l,p)
-#ifdef MKSH_DONT_EMIT_IDSTRING
-#define __IDSTRING(prefix, string)	/* nothing */
-#else
 #define __IDSTRING(prefix, string)				\
 	static const char __IDSTRING_EXPAND(__LINE__,prefix) []	\
 	    MKSH_A_USED = "@(""#)" #prefix ": " string
-#endif
 #define __COPYRIGHT(x)		__IDSTRING(copyright,x)
 #define __RCSID(x)		__IDSTRING(rcsid,x)
 #define __SCCSID(x)		__IDSTRING(sccsid,x)
 #endif
 
 #ifdef EXTERN
-__RCSID("$MirOS: src/bin/mksh/sh.h,v 1.469 2011/05/06 15:41:25 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/sh.h,v 1.373 2010/01/08 22:21:06 tg Exp $");
 #endif
-#define MKSH_VERSION "R39 2011/05/06"
+#define MKSH_VERSION "R39 2010/01/08"
 
 #ifndef MKSH_INCLUDES_ONLY
 
@@ -182,21 +179,11 @@ typedef long rlim_t;
 typedef void (*sig_t)(int);
 #endif
 
-#if !HAVE_CAN_INTTYPES
-#if !HAVE_CAN_UCBINTS
-typedef signed int int32_t;
-typedef unsigned int uint32_t;
-#else
-typedef u_int32_t uint32_t;
-#endif
-#endif
-
-#if !HAVE_CAN_INT8TYPE
-#if !HAVE_CAN_UCBINT8
-typedef unsigned char uint8_t;
-#else
-typedef u_int8_t uint8_t;
-#endif
+#if !HAVE_STDBOOL_H
+/* kludge, but enough for mksh */
+typedef int bool;
+#define false 0
+#define true 1
 #endif
 
 /* extra macros */
@@ -258,13 +245,14 @@ typedef u_int8_t uint8_t;
 #ifndef S_ISSOCK
 #define S_ISSOCK(m)	((m & 0170000) == 0140000)
 #endif
-#if !defined(S_ISCDF) && defined(S_CDF)
-#define S_ISCDF(m)	(S_ISDIR(m) && ((m) & S_CDF))
-#endif
 #ifndef DEFFILEMODE
 #define DEFFILEMODE	(S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)
 #endif
 
+#if !defined(RLIMIT_VMEM) && defined(RLIMIT_AS)
+#define RLIMIT_VMEM	RLIMIT_AS
+#endif
+
 #if !defined(MAP_FAILED)
 /* XXX imake style */
 #  if defined(__linux)
@@ -286,6 +274,16 @@ typedef u_int8_t uint8_t;
 
 /* OS-dependent additions (functions, variables, by OS) */
 
+#if !HAVE_ARC4RANDOM_DECL
+extern u_int32_t arc4random(void);
+extern void arc4random_addrandom(unsigned char *, int)
+    MKSH_A_BOUNDED(string, 1, 2);
+#endif
+
+#if !HAVE_ARC4RANDOM_PUSHB_DECL
+extern uint32_t arc4random_pushb(const void *, size_t);
+#endif
+
 #if !HAVE_FLOCK_DECL
 extern int flock(int, int);
 #endif
@@ -298,6 +296,11 @@ extern int getrusage(int, struct rusage *);
 extern int revoke(const char *);
 #endif
 
+#if !HAVE_SETMODE
+mode_t getmode(const void *, mode_t);
+void *setmode(const char *);
+#endif
+
 #ifdef __ultrix
 /* XXX imake style */
 int strcasecmp(const char *, const char *);
@@ -322,15 +325,6 @@ extern int __cdecl seteuid(uid_t);
 extern int __cdecl setegid(gid_t);
 #endif
 
-/* remove redundances */
-
-#if defined(MirBSD) && (MirBSD >= 0x08A8)
-#define MKSH_mirbsd_wcwidth
-#define utf_wcwidth(i) wcwidth((__WCHAR_TYPE__)i)
-extern int wcwidth(__WCHAR_TYPE__);
-#endif
-
-
 /* some useful #defines */
 #ifdef EXTERN
 # define I__(i) = i
@@ -350,20 +344,6 @@ typedef int32_t Tflag;
 typedef int32_t mksh_ari_t;
 typedef uint32_t mksh_uari_t;
 
-/* boolean type (no <stdbool.h> deliberately) */
-typedef unsigned char mksh_bool;
-#undef bool
-/* false MUST equal 0 */
-#undef false
-#undef true
-/* access macros for boolean type */
-#define bool		mksh_bool
-/* values must have identity mapping between mksh_bool and short */
-#define false		0
-#define true		1
-/* make any-type into bool or short */
-#define tobool(cond)	((cond) ? true : false)
-
 /* these shall be smaller than 100 */
 #ifdef MKSH_CONSERVATIVE_FDS
 #define NUFILE		32	/* Number of user-accessible files */
@@ -383,6 +363,25 @@ typedef unsigned char mksh_bool;
 
 #define LINE		4096	/* input line size */
 
+EXTERN struct {
+	const char *kshname_;	/* $0 */
+	pid_t kshpid_;		/* $$, shell PID */
+	pid_t procpid_;		/* PID of executing process */
+	pid_t kshpgrp_;		/* process group of shell */
+	uid_t ksheuid_;		/* effective UID of shell */
+	pid_t kshppid_;		/* PID of parent of shell */
+	int exstat_;		/* exit status */
+	int subst_exstat_;	/* exit status of last $(..)/`..` */
+} kshstate_;
+#define kshname		kshstate_.kshname_
+#define kshpid		kshstate_.kshpid_
+#define procpid		kshstate_.procpid_
+#define kshpgrp		kshstate_.kshpgrp_
+#define ksheuid		kshstate_.ksheuid_
+#define kshppid		kshstate_.kshppid_
+#define exstat		kshstate_.exstat_
+#define subst_exstat	kshstate_.subst_exstat_
+
 EXTERN const char *safe_prompt; /* safe prompt if PS1 substitution fails */
 EXTERN const char initvsn[] I__("KSH_VERSION=@(#)MIRBSD KSH " MKSH_VERSION);
 #define KSH_VERSION	(initvsn + /* "KSH_VERSION=@(#)" */ 16)
@@ -496,43 +495,10 @@ char *ucstrstr(char *, const char *);
 #define MKSH_S_NOVI		0
 #endif
 
-#if defined(MKSH_NOPROSPECTOFWORK) && !defined(MKSH_UNEMPLOYED)
-#define MKSH_UNEMPLOYED
-#endif
-
 /*
  * simple grouping allocator
  */
 
-
-/* 0. OS API: where to get memory from and how to free it (grouped) */
-
-/* malloc(3)/realloc(3) -> free(3) for use by the memory allocator */
-#define malloc_osi(sz)		malloc(sz)
-#define realloc_osi(p,sz)	realloc((p), (sz))
-#define free_osimalloc(p)	free(p)
-
-/* malloc(3)/realloc(3) -> free(3) for use by mksh code */
-#define malloc_osfunc(sz)	malloc(sz)
-#define realloc_osfunc(p,sz)	realloc((p), (sz))
-#define free_osfunc(p)		free(p)
-
-#if HAVE_MKNOD
-/* setmode(3) -> free(3) */
-#define free_ossetmode(p)	free(p)
-#endif
-
-#if !HAVE_MKSTEMP
-/* tempnam(3) -> free(3) */
-#define free_ostempnam(p)	free(p)
-#endif
-
-#ifdef NO_PATH_MAX
-/* GNU libc: get_current_dir_name(3) -> free(3) */
-#define free_gnu_gcdn(p)	free(p)
-#endif
-
-
 /* 1. internal structure */
 struct lalloc {
 	struct lalloc *next;
@@ -550,18 +516,6 @@ EXTERN Area aperm;		/* permanent object space */
 #define APERM	&aperm
 #define ATEMP	&e->area
 
-/*
- * flags (the order of these enums MUST match the order in misc.c(options[]))
- */
-enum sh_flag {
-#define SHFLAGS_ENUMS
-#include "sh_flags.h"
-	FNFLAGS		/* (place holder: how many flags are there) */
-};
-
-#define Flag(f)	(shell_flags[(int)(f)])
-#define UTFMODE	Flag(FUNICODE)
-
 /*
  * parsing & execution environment
  */
@@ -609,34 +563,6 @@ extern struct env {
 #define LSHELL	8	/* return to interactive shell() */
 #define LAEXPR	9	/* error in arithmetic expression */
 
-/* sort of shell global state */
-EXTERN pid_t procpid;		/* PID of executing process */
-EXTERN int exstat;		/* exit status */
-EXTERN int subst_exstat;	/* exit status of last $(..)/`..` */
-EXTERN struct tbl *vp_pipest;	/* global PIPESTATUS array */
-EXTERN short trap_exstat;	/* exit status before running a trap */
-EXTERN uint8_t trap_nested;	/* running nested traps */
-EXTERN uint8_t shell_flags[FNFLAGS];
-EXTERN const char *kshname;	/* $0 */
-EXTERN struct {
-	uid_t kshuid_;		/* real UID of shell */
-	uid_t ksheuid_;		/* effective UID of shell */
-	gid_t kshgid_;		/* real GID of shell */
-	gid_t kshegid_;		/* effective GID of shell */
-	pid_t kshpgrp_;		/* process group of shell */
-	pid_t kshppid_;		/* PID of parent of shell */
-	pid_t kshpid_;		/* $$, shell PID */
-} rndsetupstate;
-
-#define kshpid		rndsetupstate.kshpid_
-#define kshpgrp		rndsetupstate.kshpgrp_
-#define kshuid		rndsetupstate.kshuid_
-#define ksheuid		rndsetupstate.ksheuid_
-#define kshgid		rndsetupstate.kshgid_
-#define kshegid		rndsetupstate.kshegid_
-#define kshppid		rndsetupstate.kshppid_
-
-
 /* option processing */
 #define OF_CMDLINE	0x01	/* command line */
 #define OF_SET		0x02	/* set builtin */
@@ -652,34 +578,30 @@ struct shoption {
 };
 extern const struct shoption options[];
 
+/*
+ * flags (the order of these enums MUST match the order in misc.c(options[]))
+ */
+enum sh_flag {
+#define SHFLAGS_ENUMS
+#include "sh_flags.h"
+	FNFLAGS		/* (place holder: how many flags are there) */
+};
+
+#define Flag(f)	(shell_flags[(int)(f)])
+#define UTFMODE	Flag(FUNICODE)
+
+EXTERN unsigned char shell_flags[FNFLAGS];
+
 /* null value for variable; comparision pointer for unset */
 EXTERN char null[] I__("");
 /* helpers for string pooling */
-EXTERN const char T_intovfl[] I__("integer overflow %lu %c %lu prevented");
-EXTERN const char T_oomem[] I__("can't allocate %lu data bytes");
-#if defined(__GNUC__)
-/* trust this to have string pooling; -Wformat bitches otherwise */
-#define T_synerr	"syntax error"
-#else
-EXTERN const char T_synerr[] I__("syntax error");
-#endif
-EXTERN const char T_select[] I__("select");
-EXTERN const char T_r_fc_e_[] I__("r=fc -e -");
-#define T_fc_e_		(T_r_fc_e_ + 2)		/* "fc -e -" */
-#define Tn_fc_e_	7			/* strlen(T_fc_e_) */
+#define T_synerr "syntax error"
+EXTERN const char r_fc_e_[] I__("r=fc -e -");
+#define fc_e_		(r_fc_e_ + 2)		/* "fc -e -" */
+#define fc_e_n		7			/* strlen(fc_e_) */
 EXTERN const char T_local_typeset[] I__("local=typeset");
 #define T__typeset	(T_local_typeset + 5)	/* "=typeset" */
 #define T_typeset	(T_local_typeset + 6)	/* "typeset" */
-EXTERN const char T_palias[] I__("+alias");
-#define T_alias		(T_palias + 1)		/* "alias" */
-EXTERN const char T_punalias[] I__("+unalias");
-#define T_unalias	(T_punalias + 1)	/* "unalias" */
-EXTERN const char T_sgset[] I__("*=set");
-#define T_set		(T_sgset + 2)		/* "set" */
-EXTERN const char T_gbuiltin[] I__("=builtin");
-#define T_builtin	(T_gbuiltin + 1)	/* "builtin" */
-EXTERN const char T__function[] I__(" function");
-#define T_function	(T__function + 1)	/* "function" */
 
 enum temp_type {
 	TT_HEREDOC_EXP,	/* expanded heredoc */
@@ -702,7 +624,7 @@ struct temp {
 #define shl_spare	(&shf_iob[0])	/* for c_read()/c_print() */
 #define shl_stdout	(&shf_iob[1])
 #define shl_out		(&shf_iob[2])
-EXTERN bool shl_stdout_ok;
+EXTERN int shl_stdout_ok;
 
 /*
  * trap handlers
@@ -738,23 +660,16 @@ typedef struct trap {
 #define SS_RESTORE_IGN	3	/* restore to SIG_IGN */
 #define SS_FORCE	BIT(3)	/* set signal even if original signal ignored */
 #define SS_USER		BIT(4)	/* user is doing the set (ie, trap command) */
-#define SS_SHTRAP	BIT(5)	/* trap for internal use (ALRM, CHLD, WINCH) */
+#define SS_SHTRAP	BIT(5)	/* trap for internal use (CHLD,ALRM,WINCH) */
 
 #define SIGEXIT_	0	/* for trap EXIT */
 #define SIGERR_		NSIG	/* for trap ERR */
 
 EXTERN volatile sig_atomic_t trap;	/* traps pending? */
 EXTERN volatile sig_atomic_t intrsig;	/* pending trap interrupts command */
-EXTERN volatile sig_atomic_t fatal_trap; /* received a fatal signal */
+EXTERN volatile sig_atomic_t fatal_trap;/* received a fatal signal */
 extern	Trap	sigtraps[NSIG+1];
 
-/* got_winch = 1 when we need to re-adjust the window size */
-#ifdef SIGWINCH
-EXTERN volatile sig_atomic_t got_winch I__(1);
-#else
-#define got_winch	true
-#endif
-
 /*
  * TMOUT support
  */
@@ -785,9 +700,9 @@ EXTERN int really_exit;
 
 extern unsigned char chtypes[];
 
-#define ctype(c, t)	tobool( ((t) == C_SUBOP2) ?			\
+#define ctype(c, t)	!!( ((t) == C_SUBOP2) ?				\
 			    (((c) == '#' || (c) == '%') ? 1 : 0) :	\
-			    (chtypes[(unsigned char)(c)] & (t)) )
+			    (chtypes[(unsigned char)(c)]&(t)) )
 #define ksh_isalphx(c)	ctype((c), C_ALPHA)
 #define ksh_isalnux(c)	ctype((c), C_ALPHA | C_DIGIT)
 
@@ -806,13 +721,13 @@ EXTERN int ifs0 I__(' ');	/* for "$*" */
 #define GI_MINUSMINUS	BIT(2)	/* arguments were ended with -- */
 
 typedef struct {
-	const char *optarg;
-	int optind;
-	int uoptind;		/* what user sees in $OPTIND */
-	int flags;		/* see GF_* */
-	int info;		/* see GI_* */
-	unsigned int p;		/* 0 or index into argv[optind - 1] */
-	char buf[2];		/* for bad option OPTARG value */
+	const char	*optarg;
+	int		optind;
+	int		uoptind;/* what user sees in $OPTIND */
+	int		flags;	/* see GF_* */
+	int		info;	/* see GI_* */
+	unsigned int	p;	/* 0 or index into argv[optind - 1] */
+	char		buf[2];	/* for bad option OPTARG value */
 } Getopt;
 
 EXTERN Getopt builtin_opt;	/* for shell builtin commands */
@@ -831,17 +746,16 @@ struct coproc {
 };
 EXTERN struct coproc coproc;
 
-#ifndef MKSH_NOPROSPECTOFWORK
-/* used in jobs.c and by coprocess stuff in exec.c and select() calls */
+/* Used in jobs.c and by coprocess stuff in exec.c */
 EXTERN sigset_t		sm_default, sm_sigchld;
-#endif
 
 /* name of called builtin function (used by error functions) */
 EXTERN const char *builtin_argv0;
 EXTERN Tflag builtin_flag;	/* flags of called builtin (SPEC_BI, etc.) */
 
-/* current working directory */
+/* current working directory, and size of memory allocated for same */
 EXTERN char	*current_wd;
+EXTERN size_t	current_wd_size;
 
 /* Minimum required space to work with on a line - if the prompt leaves less
  * space than this on a line, the prompt is truncated.
@@ -881,11 +795,9 @@ EXTERN mksh_ari_t x_lins I__(-1);	/* tty lines */
 int shf_getc(struct shf *);
 int shf_putc(int, struct shf *);
 #else
-#define shf_getc(shf)		((shf)->rnleft > 0 ? \
-				    (shf)->rnleft--, *(shf)->rp++ : \
-				    shf_getchar(shf))
-#define shf_putc(c, shf)	((shf)->wnleft == 0 ? \
-				    shf_putchar((c), (shf)) : \
+#define shf_getc(shf) ((shf)->rnleft > 0 ? (shf)->rnleft--, *(shf)->rp++ : \
+			shf_getchar(shf))
+#define shf_putc(c, shf)	((shf)->wnleft == 0 ? shf_putchar((c), (shf)) : \
 				    ((shf)->wnleft--, *(shf)->wp++ = (c)))
 #endif
 #define shf_eof(shf)		((shf)->flags & SHF_EOF)
@@ -944,13 +856,16 @@ struct tbl {			/* table item */
 		char *s;		/* string */
 		mksh_ari_t i;		/* integer */
 		mksh_uari_t u;		/* unsigned integer */
-		int (*f)(const char **); /* int function */
+		int (*f)(const char **);/* int function */
 		struct op *t;		/* "function" tree */
 	} val;			/* value */
 	union {
 		struct tbl *array;	/* array values */
 		const char *fpath;	/* temporary path to undef function */
 	} u;
+#ifdef notyet_ktremove
+	struct table *tablep;	/* table we're ktenter'd in */
+#endif
 	union {
 		int field;	/* field with for -L/-R/-Z */
 		int errno_;	/* CEXEC/CTALIAS */
@@ -984,7 +899,7 @@ struct tbl {			/* table item */
 #define LCASEV		BIT(17)	/* convert to lower case */
 #define UCASEV_AL	BIT(18) /* convert to upper case / autoload function */
 #define INT_U		BIT(19)	/* unsigned integer */
-#define INT_L		BIT(20)	/* long integer (no-op but used as magic) */
+#define INT_L		BIT(20)	/* long integer (no-op) */
 #define IMPORT		BIT(21)	/* flag to typeset(): no arrays, must have = */
 #define LOCAL_COPY	BIT(22)	/* with LOCAL - copy attrs from existing var */
 #define EXPRINEVAL	BIT(23)	/* contents currently being evaluated */
@@ -1111,8 +1026,7 @@ struct op {
 					 */
 	int lineno;			/* TCOM/TFUNC: LINENO for this */
 	short type;			/* operation type, see below */
-	union {
-		/* WARNING: newtp(), tcopy() use evalflags = 0 to clear union */
+	union { /* WARNING: newtp(), tcopy() use evalflags = 0 to clear union */
 		short evalflags;	/* TCOM: arg expansion eval() flags */
 		short ksh_func;		/* TFUNC: function x (vs x()) */
 	} u;
@@ -1164,29 +1078,27 @@ struct op {
  * IO redirection
  */
 struct ioword {
-	int	unit;		/* unit affected */
-	int	flag;		/* action (below) */
-	char	*name;		/* file name (unused if heredoc) */
-	char	*delim;		/* delimiter for <<,<<- */
-	char	*heredoc;	/* content of heredoc */
+	int	unit;	/* unit affected */
+	int	flag;	/* action (below) */
+	char	*name;	/* file name (unused if heredoc) */
+	char	*delim;	/* delimiter for <<,<<- */
+	char	*heredoc;/* content of heredoc */
 };
 
 /* ioword.flag - type of redirection */
-#define IOTYPE		0xF	/* type: bits 0:3 */
-#define IOREAD		0x1	/* < */
-#define IOWRITE		0x2	/* > */
-#define IORDWR		0x3	/* <>: todo */
-#define IOHERE		0x4	/* << (here file) */
-#define IOCAT		0x5	/* >> */
-#define IODUP		0x6	/* <&/>& */
-#define IOEVAL		BIT(4)	/* expand in << */
-#define IOSKIP		BIT(5)	/* <<-, skip ^\t* */
-#define IOCLOB		BIT(6)	/* >|, override -o noclobber */
-#define IORDUP		BIT(7)	/* x<&y (as opposed to x>&y) */
-#define IONAMEXP	BIT(8)	/* name has been expanded */
-#define IOBASH		BIT(9)	/* &> etc. */
-#define IOHERESTR	BIT(10)	/* <<< (here string) */
-#define IONDELIM	BIT(11)	/* null delimiter (<<) */
+#define IOTYPE	0xF	/* type: bits 0:3 */
+#define IOREAD	0x1	/* < */
+#define IOWRITE	0x2	/* > */
+#define IORDWR	0x3	/* <>: todo */
+#define IOHERE	0x4	/* << (here file) */
+#define IOCAT	0x5	/* >> */
+#define IODUP	0x6	/* <&/>& */
+#define IOEVAL	BIT(4)	/* expand in << */
+#define IOSKIP	BIT(5)	/* <<-, skip ^\t* */
+#define IOCLOB	BIT(6)	/* >|, override -o noclobber */
+#define IORDUP	BIT(7)	/* x<&y (as opposed to x>&y) */
+#define IONAMEXP BIT(8)	/* name has been expanded */
+#define IOBASH	BIT(9)	/* &> etc. */
 
 /* execute/exchild flags */
 #define XEXEC	BIT(0)		/* execute without forking */
@@ -1201,7 +1113,6 @@ struct ioword {
 #define XERROK	BIT(8)		/* non-zero exit ok (for set -e) */
 #define XCOPROC BIT(9)		/* starting a co-process */
 #define XTIME	BIT(10)		/* timing TCOM command */
-#define XPIPEST	BIT(11)		/* want PIPESTATUS */
 
 /*
  * flags to control expansion of words (assumed by t->evalflags to fit
@@ -1232,7 +1143,7 @@ struct ioword {
 #define DB_BE	4	/* an inserted -BE */
 #define DB_PAT	5	/* a pattern argument */
 
-#define X_EXTRA	20	/* this many extra bytes in X string */
+#define X_EXTRA	8	/* this many extra bytes in X string */
 
 typedef struct XString {
 	char *end, *beg;	/* end, begin of string */
@@ -1261,7 +1172,7 @@ typedef char *XStringP;
 #define XcheckN(xs, xp, n) do {					\
 	int more = ((xp) + (n)) - (xs).end;			\
 	if (more > 0)						\
-		(xp) = Xcheck_grow_(&(xs), (xp), (size_t)more);	\
+		(xp) = Xcheck_grow_(&(xs), (xp), more);		\
 } while (/* CONSTCOND */ 0)
 
 /* check for overflow, expand string */
@@ -1282,7 +1193,7 @@ typedef char *XStringP;
 #define Xsavepos(xs, xp)	((xp) - (xs).beg)
 #define Xrestpos(xs, xp, n)	((xs).beg + (n))
 
-char *Xcheck_grow_(XString *, const char *, size_t);
+char *Xcheck_grow_(XString *, const char *, unsigned int);
 
 /*
  * expandable vector of generic pointers
@@ -1295,7 +1206,7 @@ typedef struct XPtrV {
 
 #define XPinit(x, n) do {					\
 	void **vp__;						\
-	vp__ = alloc2((n), sizeof(void *), ATEMP);		\
+	vp__ = alloc((n) * sizeof(void *), ATEMP);		\
 	(x).cur = (x).beg = vp__;				\
 	(x).end = vp__ + (n);					\
 } while (/* CONSTCOND */ 0)
@@ -1303,8 +1214,8 @@ typedef struct XPtrV {
 #define XPput(x, p) do {					\
 	if ((x).cur >= (x).end) {				\
 		size_t n = XPsize(x);				\
-		(x).beg = aresize2((x).beg,			\
-		    n, 2 * sizeof(void *), ATEMP);		\
+		(x).beg = aresize((x).beg,			\
+		    n * 2 * sizeof(void *), ATEMP);		\
 		(x).cur = (x).beg + n;				\
 		(x).end = (x).cur + n;				\
 	}							\
@@ -1313,7 +1224,7 @@ typedef struct XPtrV {
 
 #define XPptrv(x)	((x).beg)
 #define XPsize(x)	((x).cur - (x).beg)
-#define XPclose(x)	aresize2((x).beg, XPsize(x), sizeof(void *), ATEMP)
+#define XPclose(x)	aresize((x).beg, XPsize(x) * sizeof(void *), ATEMP)
 #define XPfree(x)	afree((x).beg, ATEMP)
 
 #define IDENT	64
@@ -1356,7 +1267,8 @@ struct source {
 #define SF_ALIAS	BIT(1)	/* faking space at end of alias */
 #define SF_ALIASEND	BIT(2)	/* faking space at end of alias */
 #define SF_TTY		BIT(3)	/* type == SSTDIN & it is a tty */
-#define SF_HASALIAS	BIT(4)	/* u.tblp valid (SALIAS, SEOF) */
+#define SF_FIRST	BIT(4)	/* initial state (to ignore UTF-8 BOM) */
+#define SF_HASALIAS	BIT(5)	/* u.tblp valid (SALIAS, SEOF) */
 
 typedef union {
 	int i;
@@ -1429,60 +1341,14 @@ EXTERN int histsize;	/* history size */
 /* user and system time of last j_waitjed job */
 EXTERN struct timeval j_usrtime, j_systime;
 
-#define notoktomul(fac1, fac2)	((fac1) && (fac2) && \
-				    (SIZE_MAX / (fac1) < (fac2)))
-#define notoktoadd(val, cnst)	((val) > (SIZE_MAX - (cnst)))
-#define checkoktoadd(val, cnst) do {					\
-	if (notoktoadd((val), (cnst)))					\
-		internal_errorf(T_intovfl, (unsigned long)(val),	\
-		    '+', (unsigned long)(cnst));			\
-} while (/* CONSTCOND */ 0)
-
-/* Bob Jenkins' one-at-a-time hash, with better start value */
-#define oaat1_init_impl(h) do {						\
-	(h) = 0x100;							\
-} while (/* CONSTCOND */ 0)
-#define oaat1_addmem_impl(h, buf, len) do {				\
-	register const uint8_t *oaat1_addmem_p = (const void *)(buf);	\
-	register size_t oaat1_addmem_n = (len);				\
-									\
-	while (oaat1_addmem_n--) {					\
-		(h) += *oaat1_addmem_p++;				\
-		(h) += (h) << 10;					\
-		(h) ^= (h) >> 6;					\
-	}								\
-} while (/* CONSTCOND */ 0)
-#define oaat1_addstr_impl(h, s) do {					\
-	register const uint8_t *oaat1_addstr_p = (const void *)(s);	\
-	register uint8_t oaat1_addstr_c;				\
-									\
-	while ((oaat1_addstr_c = *oaat1_addstr_p++)) {			\
-		h += oaat1_addstr_c;					\
-		(h) += (h) << 10;					\
-		(h) ^= (h) >> 6;					\
-	}								\
-} while (/* CONSTCOND */ 0)
-#define oaat1_fini_impl(h) do {						\
-	(h) += (h) << 3;						\
-	(h) ^= (h) >> 11;						\
-	(h) += (h) << 15;						\
-} while (/* CONSTCOND */ 0)
-
 /* lalloc.c */
 void ainit(Area *);
 void afreeall(Area *);
 /* these cannot fail and can take NULL (not for ap) */
-#define alloc(n, ap)		aresize(NULL, (n), (ap))
-#define alloc2(m, n, ap)	aresize2(NULL, (m), (n), (ap))
+#define alloc(n, ap)	aresize(NULL, (n), (ap))
 void *aresize(void *, size_t, Area *);
-void *aresize2(void *, size_t, size_t, Area *);
 void afree(void *, Area *);	/* can take NULL */
 /* edit.c */
-#ifndef MKSH_SMALL
-int x_bind(const char *, const char *, bool, bool);
-#else
-int x_bind(const char *, const char *, bool);
-#endif
 void x_init(void);
 int x_read(char *, size_t);
 /* eval.c */
@@ -1498,9 +1364,9 @@ int execute(struct op * volatile, volatile int, volatile int * volatile);
 int shcomexec(const char **);
 struct tbl *findfunc(const char *, uint32_t, bool);
 int define(const char *, struct op *);
-const char *builtin(const char *, int (*)(const char **));
+void builtin(const char *, int (*)(const char **));
 struct tbl *findcom(const char *, int);
-void flushcom(bool);
+void flushcom(int);
 const char *search(const char *, const char *, int, int *);
 int search_access(const char *, int, int *);
 int pr_menu(const char * const *);
@@ -1515,11 +1381,10 @@ int utf_widthadj(const char *, const char **);
 int utf_mbswidth(const char *);
 const char *utf_skipcols(const char *, int);
 size_t utf_ptradj(const char *);
-#ifndef MKSH_mirbsd_wcwidth
 int utf_wcwidth(unsigned int);
-#endif
 /* funcs.c */
 int c_hash(const char **);
+int c_cd(const char **);
 int c_pwd(const char **);
 int c_print(const char **);
 #ifdef MKSH_PRINTF_BUILTIN
@@ -1556,16 +1421,13 @@ int c_times(const char **);
 int timex(struct op *, int, volatile int *);
 void timex_hook(struct op *, char ** volatile *);
 int c_exec(const char **);
-/* dummy function (just need pointer value), special case in comexec() */
-#define c_builtin shcomexec
+int c_builtin(const char **);
 int c_test(const char **);
 #if HAVE_MKNOD
 int c_mknod(const char **);
 #endif
 int c_realpath(const char **);
 int c_rename(const char **);
-int c_cat(const char **);
-int c_sleep(const char **);
 /* histrap.c */
 void init_histvec(void);
 void hist_init(Source *);
@@ -1573,9 +1435,6 @@ void hist_init(Source *);
 void hist_finish(void);
 #endif
 void histsave(int *, const char *, bool, bool);
-#if !defined(MKSH_SMALL) && HAVE_PERSISTENT_HISTORY
-bool histsync(void);
-#endif
 int c_fc(const char **);
 void sethistsize(int);
 #if HAVE_PERSISTENT_HISTORY
@@ -1584,7 +1443,8 @@ void sethistfile(const char *);
 char **histpos(void);
 int histnum(int);
 int findhist(int, int, const char *, int);
-char **hist_get_newest(bool);
+int findhistrel(const char *);
+char **hist_get_newest(int);
 void inittraps(void);
 void alarm_init(void);
 Trap *gettrap(const char *, int);
@@ -1593,7 +1453,7 @@ void intrcheck(void);
 int fatal_trap_check(void);
 int trap_pending(void);
 void runtraps(int intr);
-void runtrap(Trap *, bool);
+void runtrap(Trap *);
 void cleartraps(void);
 void restoresigs(void);
 void settrap(Trap *, const char *);
@@ -1616,6 +1476,7 @@ int j_kill(const char *, int);
 int j_resume(const char *, int);
 #endif
 int j_jobs(const char *, int, int);
+int j_njobs(void);
 void j_notify(void);
 pid_t j_async(void);
 int j_stopped_running(void);
@@ -1623,14 +1484,14 @@ int j_stopped_running(void);
 int yylex(int);
 void yyerror(const char *, ...)
     MKSH_A_NORETURN
-    MKSH_A_FORMAT(__printf__, 1, 2);
+    MKSH_A_FORMAT(printf, 1, 2);
 Source *pushs(int, Area *);
 void set_prompt(int, Source *);
 void pprompt(const char *, int);
 int promptlen(const char *);
 /* main.c */
 int include(const char *, int, const char **, int);
-int command(const char *, int);
+int command(const char *);
 int shell(Source *volatile, int volatile);
 void unwind(int) MKSH_A_NORETURN;
 void newenv(int);
@@ -1639,27 +1500,25 @@ void cleanup_parents_env(void);
 void cleanup_proc_env(void);
 void errorf(const char *, ...)
     MKSH_A_NORETURN
-    MKSH_A_FORMAT(__printf__, 1, 2);
-void errorfx(int, const char *, ...)
-    MKSH_A_NORETURN
-    MKSH_A_FORMAT(__printf__, 2, 3);
+    MKSH_A_FORMAT(printf, 1, 2);
 void warningf(bool, const char *, ...)
-    MKSH_A_FORMAT(__printf__, 2, 3);
+    MKSH_A_FORMAT(printf, 2, 3);
 void bi_errorf(const char *, ...)
-    MKSH_A_FORMAT(__printf__, 1, 2);
+    MKSH_A_FORMAT(printf, 1, 2);
 #define errorfz()	errorf("\1")
-#define errorfxz(rc)	errorfx((rc), "\1")
 #define bi_errorfz()	bi_errorf("\1")
+void internal_verrorf(const char *, va_list)
+    MKSH_A_FORMAT(printf, 1, 0);
 void internal_errorf(const char *, ...)
     MKSH_A_NORETURN
-    MKSH_A_FORMAT(__printf__, 1, 2);
+    MKSH_A_FORMAT(printf, 1, 2);
 void internal_warningf(const char *, ...)
-    MKSH_A_FORMAT(__printf__, 1, 2);
+    MKSH_A_FORMAT(printf, 1, 2);
 void error_prefix(bool);
 void shellf(const char *, ...)
-    MKSH_A_FORMAT(__printf__, 1, 2);
+    MKSH_A_FORMAT(printf, 1, 2);
 void shprintf(const char *, ...)
-    MKSH_A_FORMAT(__printf__, 1, 2);
+    MKSH_A_FORMAT(printf, 1, 2);
 int can_seek(int);
 void initio(void);
 int ksh_dup2(int, int, bool);
@@ -1675,10 +1534,14 @@ void coproc_write_close(int);
 int coproc_getfd(int, const char **);
 void coproc_cleanup(int);
 struct temp *maketemp(Area *, Temp_type, struct temp **);
+#define hash(s) oaathash_full((const uint8_t *)(s))
+uint32_t oaathash_full(register const uint8_t *);
+uint32_t hashmem(const void *, size_t);
 void ktinit(struct table *, Area *, size_t);
 struct tbl *ktsearch(struct table *, const char *, uint32_t);
 struct tbl *ktenter(struct table *, const char *, uint32_t);
 #define ktdelete(p)	do { p->flag = 0; } while (/* CONSTCOND */ 0)
+void ktremove(struct tbl *);
 void ktwalk(struct tstate *, struct table *);
 struct tbl *ktnext(struct tstate *);
 struct tbl **ktsort(struct table *);
@@ -1693,23 +1556,22 @@ int getn(const char *, int *);
 int bi_getn(const char *, int *);
 int gmatchx(const char *, const char *, bool);
 int has_globbing(const char *, const char *);
+const unsigned char *pat_scan(const unsigned char *, const unsigned char *, int);
 int xstrcmp(const void *, const void *);
 void ksh_getopt_reset(Getopt *, int);
 int ksh_getopt(const char **, Getopt *, const char *);
 void print_value_quoted(const char *);
-char *quote_value(const char *);
 void print_columns(struct shf *, int,
     char *(*)(char *, int, int, const void *),
     const void *, int, int, bool);
 void strip_nuls(char *, int);
-ssize_t blocking_read(int, char *, size_t)
-    MKSH_A_BOUNDED(__buffer__, 2, 3);
+int blocking_read(int, char *, int)
+    MKSH_A_BOUNDED(buffer, 2, 3);
 int reset_nonblock(int);
-char *ksh_get_wd(void);
-char *do_realpath(const char *);
+char *ksh_get_wd(size_t *);
+int make_path(const char *, const char *, char **, XString *, int *);
 void simplify_path(char *);
-void set_current_wd(const char *);
-int c_cd(const char **);
+void set_current_wd(char *);
 #ifdef MKSH_SMALL
 char *strdup_(const char *, Area *);
 char *strndup_(const char *, size_t, Area *);
@@ -1732,36 +1594,25 @@ int shf_putchar(int, struct shf *);
 int shf_puts(const char *, struct shf *);
 int shf_write(const char *, int, struct shf *);
 int shf_fprintf(struct shf *, const char *, ...)
-    MKSH_A_FORMAT(__printf__, 2, 3);
+    MKSH_A_FORMAT(printf, 2, 3);
 int shf_snprintf(char *, int, const char *, ...)
-    MKSH_A_FORMAT(__printf__, 3, 4)
-    MKSH_A_BOUNDED(__string__, 1, 2);
+    MKSH_A_FORMAT(printf, 3, 4)
+    MKSH_A_BOUNDED(string, 1, 2);
 char *shf_smprintf(const char *, ...)
-    MKSH_A_FORMAT(__printf__, 1, 2);
+    MKSH_A_FORMAT(printf, 1, 2);
 int shf_vfprintf(struct shf *, const char *, va_list)
-    MKSH_A_FORMAT(__printf__, 2, 0);
+    MKSH_A_FORMAT(printf, 2, 0);
 /* syn.c */
 void initkeywords(void);
-struct op *compile(Source *, bool);
-bool parse_usec(const char *, struct timeval *);
-char *yyrecursive(void);
+struct op *compile(Source *);
 /* tree.c */
-void fptreef(struct shf *, int, const char *, ...);
+int fptreef(struct shf *, int, const char *, ...);
 char *snptreef(char *, int, const char *, ...);
 struct op *tcopy(struct op *, Area *);
 char *wdcopy(const char *, Area *);
 const char *wdscan(const char *, int);
-#define WDS_TPUTS	BIT(0)		/* tputS (dumpwdvar) mode */
-#define WDS_KEEPQ	BIT(1)		/* keep quote characters */
-#define WDS_MAGIC	BIT(2)		/* make MAGIC */
-char *wdstrip(const char *, int);
+char *wdstrip(const char *, bool, bool);
 void tfree(struct op *, Area *);
-void dumpchar(struct shf *, int);
-void dumptree(struct shf *, struct op *);
-void dumpwdvar(struct shf *, const char *);
-void vistree(char *, size_t, struct op *)
-    MKSH_A_BOUNDED(__string__, 1, 2);
-void fpFUNCTf(struct shf *, int, bool, const char *, struct op *);
 /* var.c */
 void newblock(void);
 void popblock(void);
@@ -1772,20 +1623,20 @@ char *str_val(struct tbl *);
 int setstr(struct tbl *, const char *, int);
 struct tbl *setint_v(struct tbl *, struct tbl *, bool);
 void setint(struct tbl *, mksh_ari_t);
-struct tbl *typeset(const char *, Tflag, Tflag, int, int)
-    MKSH_A_NONNULL((__nonnull__ (1)));
+struct tbl *typeset(const char *, Tflag, Tflag, int, int);
 void unset(struct tbl *, int);
 const char *skip_varname(const char *, int);
 const char *skip_wdvarname(const char *, int);
 int is_wdvarname(const char *, int);
 int is_wdvarassign(const char *);
 char **makenv(void);
+#if !HAVE_ARC4RANDOM
+void change_random(const void *, size_t);
+#endif
 void change_winsz(void);
 int array_ref_len(const char *);
 char *arrayname(const char *);
 mksh_uari_t set_array(const char *, bool, const char **);
-uint32_t hash(const void *);
-void rndset(long);
 
 enum Test_op {
 	TO_NONOP = 0,	/* non-operator */
@@ -1798,9 +1649,7 @@ enum Test_op {
 	TO_FILRD, TO_FILGZ, TO_FILTT, TO_FILSETU, TO_FILWR, TO_FILEX,
 	/* binary operators */
 	TO_STEQL, TO_STNEQ, TO_STLT, TO_STGT, TO_INTEQ, TO_INTNE, TO_INTGT,
-	TO_INTGE, TO_INTLT, TO_INTLE, TO_FILEQ, TO_FILNT, TO_FILOT,
-	/* not an operator */
-	TO_NONNULL	/* !TO_NONOP */
+	TO_INTGE, TO_INTLT, TO_INTLE, TO_FILEQ, TO_FILNT, TO_FILOT
 };
 typedef enum Test_op Test_op;
 
@@ -1820,18 +1669,19 @@ typedef enum Test_meta Test_meta;
 #define TEF_ERROR	BIT(0)		/* set if we've hit an error */
 #define TEF_DBRACKET	BIT(1)		/* set if [[ .. ]] test */
 
-typedef struct test_env {
+typedef struct test_env Test_env;
+struct test_env {
 	union {
-		const char **wp;	/* used by ptest_* */
-		XPtrV *av;		/* used by dbtestp_* */
+		const char **wp;/* used by ptest_* */
+		XPtrV *av;	/* used by dbtestp_* */
 	} pos;
-	const char **wp_end;		/* used by ptest_* */
-	Test_op (*isa)(struct test_env *, Test_meta);
-	const char *(*getopnd) (struct test_env *, Test_op, bool);
-	int (*eval)(struct test_env *, Test_op, const char *, const char *, bool);
-	void (*error)(struct test_env *, int, const char *);
-	int flags;			/* TEF_* */
-} Test_env;
+	const char **wp_end;	/* used by ptest_* */
+	int (*isa)(Test_env *, Test_meta);
+	const char *(*getopnd) (Test_env *, Test_op, bool);
+	int (*eval)(Test_env *, Test_op, const char *, const char *, bool);
+	void (*error)(Test_env *, int, const char *);
+	int flags;		/* TEF_* */
+};
 
 extern const char *const dbtest_tokens[];
 
@@ -1840,7 +1690,7 @@ int test_eval(Test_env *, Test_op, const char *, const char *, bool);
 int test_parse(Test_env *);
 
 EXTERN int tty_fd I__(-1);	/* dup'd tty file descriptor */
-EXTERN bool tty_devtty;		/* true if tty_fd is from /dev/tty */
+EXTERN int tty_devtty;		/* true if tty_fd is from /dev/tty */
 EXTERN struct termios tty_state;	/* saved tty state */
 
 extern void tty_init(bool, bool);
