@@ -1,7 +1,7 @@
 /*	$OpenBSD: var.c,v 1.34 2007/10/15 02:16:35 deraadt Exp $	*/
 
 /*-
- * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+ * Copyright (c) 2003, 2004, 2005, 2006, 2007, 2008, 2009
  *	Thorsten Glaser <tg@mirbsd.org>
  *
  * Provided that these terms and disclaimer and all copyright notices
@@ -22,11 +22,7 @@
 
 #include "sh.h"
 
-#if defined(__OpenBSD__)
-#include <sys/sysctl.h>
-#endif
-
-__RCSID("$MirOS: src/bin/mksh/var.c,v 1.120 2011/05/04 23:16:05 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/var.c,v 1.100 2009/12/05 17:43:50 tg Exp $");
 
 /*
  * Variables
@@ -39,10 +35,8 @@ __RCSID("$MirOS: src/bin/mksh/var.c,v 1.120 2011/05/04 23:16:05 tg Exp $");
  */
 static struct tbl vtemp;
 static struct table specials;
-static uint32_t lcg_state = 5381;
-
 static char *formatstr(struct tbl *, const char *);
-static void exportprep(struct tbl *, const char *);
+static void export(struct tbl *, const char *);
 static int special(const char *);
 static void unspecial(const char *);
 static void getspec(struct tbl *);
@@ -52,6 +46,12 @@ static int getint(struct tbl *, mksh_ari_t *, bool);
 static mksh_ari_t intval(struct tbl *);
 static struct tbl *arraysearch(struct tbl *, uint32_t);
 static const char *array_index_calc(const char *, bool *, uint32_t *);
+#if !HAVE_ARC4RANDOM
+static uint32_t oaathash_update(register uint32_t, register const uint8_t *,
+    register size_t);
+
+static uint32_t lcg_state = 5381;
+#endif
 
 uint8_t set_refflag = 0;
 
@@ -392,7 +392,7 @@ intval(struct tbl *vp)
 	base = getint(vp, &num, false);
 	if (base == -1)
 		/* XXX check calls - is error here ok by POSIX? */
-		errorf("%s: %s", str_val(vp), "bad number");
+		errorf("%s: bad number", str_val(vp));
 	return (num);
 }
 
@@ -405,9 +405,9 @@ setstr(struct tbl *vq, const char *s, int error_ok)
 
 	error_ok &= ~0x4;
 	if ((vq->flag & RDONLY) && !no_ro_check) {
-		warningf(true, "%s: %s", vq->name, "is read only");
+		warningf(true, "%s: is read only", vq->name);
 		if (!error_ok)
-			errorfxz(2);
+			errorfz();
 		return (0);
 	}
 	if (!(vq->flag&INTEGER)) { /* string dest */
@@ -425,7 +425,7 @@ setstr(struct tbl *vq, const char *s, int error_ok)
 		if (s && (vq->flag & (UCASEV_AL|LCASEV|LJUST|RJUST)))
 			s = salloc = formatstr(vq, s);
 		if ((vq->flag&EXPORT))
-			exportprep(vq, s);
+			export(vq, s);
 		else {
 			strdupx(vq->val.s, s, vq->areap);
 			vq->flag |= ALLOC;
@@ -644,17 +644,14 @@ formatstr(struct tbl *vp, const char *s)
  * make vp->val.s be "name=value" for quick exporting.
  */
 static void
-exportprep(struct tbl *vp, const char *val)
+export(struct tbl *vp, const char *val)
 {
 	char *xp;
 	char *op = (vp->flag&ALLOC) ? vp->val.s : NULL;
-	size_t namelen, vallen;
-
-	namelen = strlen(vp->name);
-	vallen = strlen(val) + 1;
+	int namelen = strlen(vp->name);
+	int vallen = strlen(val) + 1;
 
 	vp->flag |= ALLOC;
-	/* since name+val are both in memory this can go unchecked */
 	xp = alloc(namelen + 1 + vallen, vp->areap);
 	memcpy(vp->val.s = xp, vp->name, namelen);
 	xp += namelen;
@@ -677,19 +674,17 @@ typeset(const char *var, Tflag set, Tflag clr, int field, int base)
 	struct tbl *vpbase, *t;
 	char *tvar;
 	const char *val;
-	int len;
-	bool vappend = false;
 
 	/* check for valid variable name, search for value */
 	val = skip_varname(var, false);
 	if (val == var)
 		return (NULL);
-	mkssert(var != NULL);
-	mkssert(*var != 0);
 	if (*val == '[') {
+		int len;
+
 		if (set_refflag)
-			errorf("%s: %s", var,
-			    "reference variable can't be an array");
+			errorf("%s: reference variable cannot be an array",
+			    var);
 		len = array_ref_len(val);
 		if (len == 0)
 			return (NULL);
@@ -707,31 +702,22 @@ typeset(const char *var, Tflag set, Tflag clr, int field, int base)
 		}
 		val += len;
 	}
-	if (val[0] == '=' || (val[0] == '+' && val[1] == '=')) {
-		strndupx(tvar, var, val - var, ATEMP);
-		if (*val++ == '+') {
-			++val;
-			vappend = true;
-		}
-	} else {
+	if (*val == '=')
+		strndupx(tvar, var, val++ - var, ATEMP);
+	else {
 		/* Importing from original environment: must have an = */
 		if (set & IMPORT)
 			return (NULL);
 		strdupx(tvar, var, ATEMP);
 		val = NULL;
-		/* handle foo[*] => foo (whole array) mapping for R39b */
-		len = strlen(tvar);
-		if (len > 3 && tvar[len-3] == '[' && tvar[len-2] == '*' &&
-		    tvar[len-1] == ']')
-			tvar[len-3] = '\0';
 	}
 
 	/* Prevent typeset from creating a local PATH/ENV/SHELL */
 	if (Flag(FRESTRICTED) && (strcmp(tvar, "PATH") == 0 ||
 	    strcmp(tvar, "ENV") == 0 || strcmp(tvar, "SHELL") == 0))
-		errorf("%s: %s", tvar, "restricted");
+		errorf("%s: restricted", tvar);
 
-	vp = (set&LOCAL) ? local(tvar, tobool(set & LOCAL_COPY)) :
+	vp = (set&LOCAL) ? local(tvar, (set & LOCAL_COPY) ? true : false) :
 	    global(tvar);
 	if (set_refflag == 2 && (vp->flag & (ARRAY|ASSOC)) == ASSOC)
 		vp->flag &= ~ASSOC;
@@ -757,26 +743,23 @@ typeset(const char *var, Tflag set, Tflag clr, int field, int base)
 
 	vpbase = (vp->flag & ARRAY) ? global(arrayname(var)) : vp;
 
-	/*
-	 * only allow export flag to be set; AT&T ksh allows any
-	 * attribute to be changed which means it can be truncated or
-	 * modified (-L/-R/-Z/-i)
+	/* only allow export flag to be set. AT&T ksh allows any attribute to
+	 * be changed which means it can be truncated or modified (-L/-R/-Z/-i)
 	 */
 	if ((vpbase->flag&RDONLY) &&
 	    (val || clr || (set & ~EXPORT)))
 		/* XXX check calls - is error here ok by POSIX? */
-		errorfx(2, "%s: %s", tvar, "is read only");
+		errorf("%s: is read only", tvar);
 	afree(tvar, ATEMP);
 
 	/* most calls are with set/clr == 0 */
 	if (set | clr) {
-		bool ok = true;
-
+		int ok = 1;
 		/* XXX if x[0] isn't set, there will be problems: need to have
 		 * one copy of attributes for arrays...
 		 */
 		for (t = vpbase; t; t = t->u.array) {
-			bool fake_assign;
+			int fake_assign;
 			char *s = NULL;
 			char *free_me = NULL;
 
@@ -800,9 +783,8 @@ typeset(const char *var, Tflag set, Tflag clr, int field, int base)
 				t->flag &= ~ALLOC;
 			}
 			t->flag = (t->flag | set) & ~clr;
-			/*
-			 * Don't change base if assignment is to be
-			 * done, in case assignment fails.
+			/* Don't change base if assignment is to be done,
+			 * in case assignment fails.
 			 */
 			if ((set & INTEGER) && base > 0 && (!val || t != vp))
 				t->type = base;
@@ -810,13 +792,11 @@ typeset(const char *var, Tflag set, Tflag clr, int field, int base)
 				t->u2.field = field;
 			if (fake_assign) {
 				if (!setstr(t, s, KSH_RETURN_ERROR)) {
-					/*
-					 * Somewhat arbitrary action
-					 * here: zap contents of
-					 * variable, but keep the flag
-					 * settings.
+					/* Somewhat arbitrary action here:
+					 * zap contents of variable, but keep
+					 * the flag settings.
 					 */
-					ok = false;
+					ok = 0;
 					if (t->flag & INTEGER)
 						t->flag &= ~ISSET;
 					else {
@@ -835,14 +815,6 @@ typeset(const char *var, Tflag set, Tflag clr, int field, int base)
 	}
 
 	if (val != NULL) {
-		char *tval;
-
-		if (vappend) {
-			tval = shf_smprintf("%s%s", str_val(vp), val);
-			val = tval;
-		} else
-			tval = NULL;
-
 		if (vp->flag&INTEGER) {
 			/* do not zero base before assignment */
 			setstr(vp, val, KSH_UNWIND_ERROR | 0x4);
@@ -852,30 +824,25 @@ typeset(const char *var, Tflag set, Tflag clr, int field, int base)
 		} else
 			/* setstr can't fail (readonly check already done) */
 			setstr(vp, val, KSH_RETURN_ERROR | 0x4);
-
-		if (tval != NULL)
-			afree(tval, ATEMP);
 	}
 
 	/* only x[0] is ever exported, so use vpbase */
 	if ((vpbase->flag&EXPORT) && !(vpbase->flag&INTEGER) &&
 	    vpbase->type == 0)
-		exportprep(vpbase, (vpbase->flag&ISSET) ? vpbase->val.s : null);
+		export(vpbase, (vpbase->flag&ISSET) ? vpbase->val.s : null);
 
 	return (vp);
 }
 
-/**
- * Unset a variable. The flags can be:
- * |1	= tear down entire array
- * |2	= keep attributes, only unset content
+/* Unset a variable. array_ref is set if there was an array reference in
+ * the name lookup (eg, x[2]).
  */
 void
-unset(struct tbl *vp, int flags)
+unset(struct tbl *vp, int array_ref)
 {
 	if (vp->flag & ALLOC)
 		afree(vp->val.s, vp->areap);
-	if ((vp->flag & ARRAY) && (flags & 1)) {
+	if ((vp->flag & ARRAY) && !array_ref) {
 		struct tbl *a, *tmp;
 
 		/* Free up entire array */
@@ -888,12 +855,8 @@ unset(struct tbl *vp, int flags)
 		}
 		vp->u.array = NULL;
 	}
-	if (flags & 2) {
-		vp->flag &= ~(ALLOC|ISSET);
-		return;
-	}
 	/* If foo[0] is being unset, the remainder of the array is kept... */
-	vp->flag &= SPECIAL | ((flags & 1) ? 0 : ARRAY|DEFINED);
+	vp->flag &= SPECIAL | (array_ref ? ARRAY|DEFINED : 0);
 	if (vp->flag & SPECIAL)
 		unsetspec(vp);	/* responsible for 'unspecial'ing var */
 }
@@ -931,7 +894,7 @@ skip_wdvarname(const char *s,
 			char c;
 			int depth = 0;
 
-			while (/* CONSTCOND */ 1) {
+			while (1) {
 				if (p[0] != CHAR)
 					break;
 				c = p[1];
@@ -963,8 +926,7 @@ is_wdvarassign(const char *s)
 {
 	const char *p = skip_wdvarname(s, true);
 
-	return (p != s && p[0] == CHAR &&
-	    (p[1] == '=' || (p[1] == '+' && p[2] == CHAR && p[3] == '=')));
+	return (p != s && p[0] == CHAR && p[1] == '=');
 }
 
 /*
@@ -1007,6 +969,48 @@ makenv(void)
 	return ((char **)XPclose(denv));
 }
 
+#if !HAVE_ARC4RANDOM
+static uint32_t
+oaathash_update(register uint32_t h, register const uint8_t *cp,
+    register size_t n)
+{
+	while (n--) {
+		h += *cp++;
+		h += h << 10;
+		h ^= h >> 6;
+	}
+
+	return (h);
+}
+
+void
+change_random(const void *vp, size_t n)
+{
+	register uint32_t h = 0x100;
+	struct {
+		const void *sp, *bp, *dp;
+		size_t dsz;
+		struct timeval tv;
+		uint32_t s;
+	} i;
+
+	i.dp = vp;
+	i.dsz = n;
+	i.s = lcg_state;
+	i.bp = &lcg_state;
+	i.sp = &i;
+	gettimeofday(&i.tv, NULL);
+	h = oaathash_update(oaathash_update(h, (void *)&i, sizeof(i)), vp, n);
+
+	/* oaathash_finalise */
+	h += h << 3;
+	h ^= h >> 11;
+	h += h << 15;
+
+	lcg_state = h;
+}
+#endif
+
 /*
  * handle special variables with side effects - PATH, SECONDS.
  */
@@ -1057,11 +1061,15 @@ getspec(struct tbl *vp)
 			return;
 		break;
 	case V_RANDOM:
+#if HAVE_ARC4RANDOM
+		i = arc4random() & 0x7FFF;
+#else
 		/*
 		 * this is the same Linear Congruential PRNG as Borland
 		 * C/C++ allegedly uses in its built-in rand() function
 		 */
 		i = ((lcg_state = 22695477 * lcg_state + 1) >> 16) & 0x7FFF;
+#endif
 		break;
 	case V_HISTSIZE:
 		i = histsize;
@@ -1106,8 +1114,7 @@ setspec(struct tbl *vp)
 			afree(path, APERM);
 		s = str_val(vp);
 		strdupx(path, s, APERM);
-		/* clear tracked aliases */
-		flushcom(true);
+		flushcom(1);	/* clear tracked aliases */
 		return;
 	case V_IFS:
 		setctypes(s = str_val(vp), C_IFS);
@@ -1129,17 +1136,17 @@ setspec(struct tbl *vp)
 			    stat(s, &statb) == 0 && S_ISDIR(statb.st_mode))
 				strdupx(tmpdir, s, APERM);
 		}
-		return;
+		break;
 #if HAVE_PERSISTENT_HISTORY
 	case V_HISTFILE:
 		sethistfile(str_val(vp));
-		return;
+		break;
 #endif
 	case V_TMOUT:
 		/* AT&T ksh seems to do this (only listen if integer) */
 		if (vp->flag & INTEGER)
 			ksh_tmout = vp->val.i >= 0 ? vp->val.i : 0;
-		return;
+		break;
 
 	/* common sub-cases */
 	case V_OPTIND:
@@ -1176,11 +1183,25 @@ setspec(struct tbl *vp)
 			x_lins = i;
 		break;
 	case V_RANDOM:
+#if HAVE_ARC4RANDOM
+#if HAVE_ARC4RANDOM_PUSHB
+		if (Flag(FARC4RANDOM))
+			/*
+			 * things like initialisation, environment import,
+			 * etc. are already done
+			 */
+			arc4random_pushb(&i, sizeof(i));
+		else
+			/* during start-up phase or somesuch */
+#endif	/* HAVE_ARC4RANDOM_PUSHB */
+			arc4random_addrandom((void *)&i, sizeof(i));
+#else	/* !HAVE_ARC4RANDOM */
 		/*
-		 * mksh R39d+ no longer has the traditional repeatability
+		 * mksh R40+ no longer has the traditional repeatability
 		 * of $RANDOM sequences, but always retains state
 		 */
-		rndset((long)i);
+		change_random(&i, sizeof(i));
+#endif	/* !HAVE_ARC4RANDOM */
 		break;
 	case V_SECONDS:
 		{
@@ -1205,8 +1226,7 @@ unsetspec(struct tbl *vp)
 		if (path)
 			afree(path, APERM);
 		strdupx(path, def_path, APERM);
-		/* clear tracked aliases */
-		flushcom(true);
+		flushcom(1);	/* clear tracked aliases */
 		break;
 	case V_IFS:
 		setctypes(" \t\n", C_IFS);
@@ -1226,12 +1246,13 @@ unsetspec(struct tbl *vp)
 		unspecial(vp->name);
 		break;
 
-	/*
-	 * AT&T ksh man page says OPTIND, OPTARG and _ lose special
-	 * meaning, but OPTARG does not (still set by getopts) and _ is
-	 * also still set in various places. Don't know what AT&T does
-	 * for HISTSIZE, HISTFILE. Unsetting these in AT&T ksh does not
-	 * loose the 'specialness': IFS, COLUMNS, PATH, TMPDIR
+	/* AT&T ksh man page says OPTIND, OPTARG and _ lose special meaning,
+	 * but OPTARG does not (still set by getopts) and _ is also still
+	 * set in various places.
+	 * Don't know what AT&T does for:
+	 *		HISTSIZE, HISTFILE,
+	 * Unsetting these in AT&T ksh does not loose the 'specialness':
+	 * no effect: IFS, COLUMNS, PATH, TMPDIR
 	 */
 	}
 }
@@ -1243,7 +1264,7 @@ unsetspec(struct tbl *vp)
 static struct tbl *
 arraysearch(struct tbl *vp, uint32_t val)
 {
-	struct tbl *prev, *curr, *news;
+	struct tbl *prev, *curr, *new;
 	size_t len;
 
 	vp->flag = (vp->flag | (ARRAY|DEFINED)) & ~ASSOC;
@@ -1259,26 +1280,29 @@ arraysearch(struct tbl *vp, uint32_t val)
 	if (curr && curr->ua.index == val) {
 		if (curr->flag&ISSET)
 			return (curr);
-		news = curr;
+		new = curr;
 	} else
-		news = NULL;
-	if (!news) {
-		len = strlen(vp->name);
-		checkoktoadd(len, 1 + offsetof(struct tbl, name[0]));
-		news = alloc(offsetof(struct tbl, name[0]) + ++len, vp->areap);
-		memcpy(news->name, vp->name, len);
+		new = NULL;
+	len = strlen(vp->name) + 1;
+	if (!new) {
+		new = alloc(offsetof(struct tbl, name[0]) + len, vp->areap);
+		memcpy(new->name, vp->name, len);
 	}
-	news->flag = (vp->flag & ~(ALLOC|DEFINED|ISSET|SPECIAL)) | AINDEX;
-	news->type = vp->type;
-	news->areap = vp->areap;
-	news->u2.field = vp->u2.field;
-	news->ua.index = val;
+	new->flag = (vp->flag & ~(ALLOC|DEFINED|ISSET|SPECIAL)) | AINDEX;
+	new->type = vp->type;
+	new->areap = vp->areap;
+	new->u2.field = vp->u2.field;
+	new->ua.index = val;
+#ifdef notyet_ktremove
+	/* XXX array indices must not be ktdelete'd, for now */
+	new->tablep = NULL;
+#endif
 
-	if (curr != news) {		/* not reusing old array entry */
-		prev->u.array = news;
-		news->u.array = curr;
+	if (curr != new) {		/* not reusing old array entry */
+		prev->u.array = new;
+		new->u.array = curr;
 	}
-	return (news);
+	return (new);
 }
 
 /* Return the length of an array reference (eg, [1+2]) - cp is assumed
@@ -1323,42 +1347,33 @@ mksh_uari_t
 set_array(const char *var, bool reset, const char **vals)
 {
 	struct tbl *vp, *vq;
-	mksh_uari_t i, j = 0;
+	mksh_uari_t i;
 	const char *ccp;
-	char *cp = NULL;
+#ifndef MKSH_SMALL
+	char *cp;
+	mksh_uari_t j;
+#endif
 
 	/* to get local array, use "typeset foo; set -A foo" */
-	i = strlen(var);
-	if (i > 0 && var[i - 1] == '+') {
-		/* append mode */
-		reset = false;
-		strndupx(cp, var, i - 1, ATEMP);
-	}
-	vp = global(cp ? cp : var);
+	vp = global(var);
 
 	/* Note: AT&T ksh allows set -A but not set +A of a read-only var */
 	if ((vp->flag&RDONLY))
-		errorfx(2, "%s: %s", cp ? cp : var, "is read only");
+		errorf("%s: is read only", var);
 	/* This code is quite non-optimal */
 	if (reset)
 		/* trash existing values and attributes */
-		unset(vp, 1);
-	/*
-	 * todo: would be nice for assignment to completely succeed or
+		unset(vp, 0);
+	/* todo: would be nice for assignment to completely succeed or
 	 * completely fail. Only really effects integer arrays:
 	 * evaluation of some of vals[] may fail...
 	 */
 	i = 0;
-	if (cp != NULL) {
-		/* find out where to set when appending */
-		for (vq = vp; vq; vq = vq->u.array) {
-			if (!(vq->flag & ISSET))
-				continue;
-			if (arrayindex(vq) >= j)
-				j = arrayindex(vq) + 1;
-		}
-		afree(cp, ATEMP);
-	}
+#ifndef MKSH_SMALL
+	j = 0;
+#else
+#define j i
+#endif
 	while ((ccp = vals[i])) {
 #ifndef MKSH_SMALL
 		if (*ccp == '[') {
@@ -1385,9 +1400,15 @@ set_array(const char *var, bool reset, const char **vals)
 
 		vq = arraysearch(vp, j);
 		/* would be nice to deal with errors here... (see above) */
+#if 0
+		shprintf("setting '%s'[%lu]='%s'  <- '%s'\n",
+		    vp->name, (unsigned long)j, ccp, vals[i]);
+#endif
 		setstr(vq, ccp, KSH_RETURN_ERROR);
 		i++;
+#ifndef MKSH_SMALL
 		j++;
+#endif
 	}
 
 	return (i);
@@ -1407,7 +1428,7 @@ change_winsz(void)
 	}
 
 #ifdef TIOCGWINSZ
-	/* check if window size has changed */
+	/* check if window size has changed since first time */
 	if (tty_fd >= 0) {
 		struct winsize ws;
 
@@ -1425,49 +1446,4 @@ change_winsz(void)
 		x_cols = 80;
 	if (x_lins < MIN_LINS)
 		x_lins = 24;
-
-#ifdef SIGWINCH
-	got_winch = 0;
-#endif
-}
-
-uint32_t
-hash(const void *s)
-{
-	register uint32_t h;
-
-	oaat1_init_impl(h);
-	oaat1_addstr_impl(h, s);
-	oaat1_fini_impl(h);
-	return (h);
-}
-
-void
-rndset(long v)
-{
-	register uint32_t h;
-
-	oaat1_init_impl(h);
-	oaat1_addmem_impl(h, &lcg_state, sizeof(lcg_state));
-	oaat1_addmem_impl(h, &v, sizeof(v));
-
-#if defined(arc4random_pushb_fast) || defined(MKSH_A4PB)
-	/*
-	 * either we have very chap entropy get and push available,
-	 * with malloc() pulling in this code already anyway, or the
-	 * user requested us to use the old functions
-	 */
-	lcg_state = h;
-	oaat1_fini_impl(lcg_state);
-#if defined(arc4random_pushb_fast)
-	arc4random_pushb_fast(&lcg_state, sizeof(lcg_state));
-	lcg_state = arc4random();
-#else
-	lcg_state = arc4random_pushb(&lcg_state, sizeof(lcg_state));
-#endif
-	oaat1_addmem_impl(h, &lcg_state, sizeof(lcg_state));
-#endif
-
-	oaat1_fini_impl(h);
-	lcg_state = h;
 }
